# # -*- coding: utf-8 -*-
# from django.db import models
# from django.utils.translation import ugettext_lazy as _
# from django.contrib.contenttypes import generic
# from django.contrib.contenttypes.models import ContentType
# from django_extensions.db import fields as exfields

# class BaseSemLink(models.Model):
#     """Linked Data generic link model, can apply to SeeAlso or SameAs"""
#     url             = models.URLField(blank=True, verify_exists=True, verbose_name=_(u'URL'))
#     label           = models.CharField(_(u'Label'),max_length=100)
#     slug            = exfields.AutoSlugField(populate_from=('label'))
#     created         = exfields.CreationDateTimeField(_(u'created'),null=True)
#     modified        = exfields.ModificationDateTimeField(_(u'modified'),null=True)
#     content_type    = models.ForeignKey(ContentType, blank=True, null=True)
#     object_id       = models.PositiveIntegerField()
#     content_object  = generic.GenericForeignKey('content_type', 'object_id')
    
#     # TODO et author uri etc ?
    
#     def __unicode__(self):
#         return self.label
#     class Meta :
#         abstract = True
        
        




# -*- coding:utf-8 -*-
from django.contrib.auth.models import User
from django import forms
from django.utils.translation import ugettext_lazy as _
from coop_local.models import Person

# Repiquage carrément du modèle
attrs_dict = { 'class': 'required' }

# from http://birdhouse.org/blog/2009/06/27/django-profiles/
class ProfileForm(forms.ModelForm):
  
    last_name = forms.CharField(max_length=30,
                                widget=forms.TextInput(attrs=attrs_dict),
                                label=_(u'name'),
                                )
    username = forms.RegexField(regex=r'^\w+$',
                                max_length=30,
                                widget=forms.TextInput(attrs=attrs_dict),
                                label=_("username"),
                                error_messages={ 'invalid': _("This value must contain only letters, numbers and underscores.") },
                                help_text=u"Ce nom d'utilisateur sera public, affiché à la place du nom pour les visiteurs anonymes du site (Google par exemple)"
                                )
    email = forms.EmailField(widget=forms.TextInput(attrs=dict(attrs_dict,
                                                               maxlength=75)),
                             label=_("email address"),
                             help_text= _(u"your personal email")
                             )
                             
    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)
        try:
            self.fields['email'].initial = self.instance.user.email
            self.fields['last_name'].initial = self.instance.user.last_name
            self.fields['username'].initial = self.instance.user.username
        except User.DoesNotExist:
            pass
    
        self.fields.keyOrder = ['last_name','username','email', 'adresse','code_postal','ville','telephone_fixe','telephone_portable']
            
    class Meta:
        model = Person
        exclude = ('user',)
    def save(self, *args, **kwargs):
        """
        Update the primary email address on the related User object as well.
        """
        u = self.instance.user
        u.email = self.cleaned_data['email']
        u.last_name = self.cleaned_data['last_name']
        u.username = self.cleaned_data['username']
        u.save()
        profile = super(ProfileForm, self).save(*args,**kwargs)
        return profile

    


# -*- coding:utf-8 -*-

from django.shortcuts import render_to_response, redirect
from coop_local.models import Organization, Person
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.conf import settings
from django.http import Http404
from django.http import HttpResponse
from django.shortcuts import get_object_or_404

if('coop.exchange' in settings.INSTALLED_APPS):
    from coop_local.models import Exchange


def home(request):
    rdict = {}
    rdict['dernieres_initiatives'] = Organization.objects.filter(active=True)[:10]
    if('coop.exchange' in settings.INSTALLED_APPS):
        rdict['dernieres_annonces'] = Exchange.objects.all()[:10]
    return render_to_response('home.html', rdict, RequestContext(request))


def d2r_mapping(request, mode):
    if(request.META['REMOTE_ADDR'] in settings.INTERNAL_IPS and mode in ['view', 'export']):
        db = settings.DATABASES['default']
        rdict = {}
        rdict['mode'] = mode
        rdict['namespaces'] = settings.RDF_NAMESPACES
        rdict['d2r_baseURI'] = settings.D2RQ_ROOT
        rdict['d2r_niceURI'] = 'http://' + settings.DEFAULT_URI_DOMAIN + '/'
        rdict['d2r_port'] = settings.D2RQ_PORT
        rdict['d2r_site_name'] = unicode(settings.SITE_TITLE) + u' : RDF mapping'
        rdict['d2rq_username'] = db['USER']
        rdict['d2rq_password'] = db['PASSWORD']
        rdict['d2rq_db_name'] = db['NAME']
        rdict['d2rq_db_port'] = db['PORT'] if db['PORT'] != '' else 5432
        rdict['d2rq_db_host'] = db['HOST'] if db['HOST'] != '' else 'localhost'
    else:
        raise Http404
    return render_to_response('d2r/mapping.ttl', rdict, RequestContext(request))


from django.db import models
from coop.models import StaticURIModel
from django.db.models.loading import get_model

urimodels = dict((m.__name__.lower(), m.__module__.split('.')[0]) for m in models.get_models() if m.__mro__.__contains__(StaticURIModel))

RDF_SERIALIZATIONS = {
    'nt': 'text/plain',
    'n3': 'text/n3',
    'ttl': 'text/turtle',
    'xml': 'application/rdf+xml',
    'json': 'application/json'
}


def get_rdf(request, model, uuid, format):
    req_model = get_model(urimodels[model], model)
    object = get_object_or_404(req_model, uuid=uuid)
    return HttpResponse(object.toRdf(format), mimetype=RDF_SERIALIZATIONS[format])


def SentryHandler500(request):
    from django.template import Context, loader
    from django.http import HttpResponseServerError

    t = loader.get_template('500.html')  # You need to create a 500.html template.
    return HttpResponseServerError(t.render(Context({
        'request': request,
        'STATIC_URL': settings.STATIC_URL,
    })))




import logging

from django.contrib.auth.models import UserManager
from django.db import models

logger = logging.getLogger(name=__name__)


def create_blank_profile(user=None):
    from django_webid.provider.models import WebIDUser
    from django.conf import settings
    if not user:
        logger.warning('create_blank_profile: no user.')
        return
    profile_model = settings.AUTH_PROFILE_MODULE
    app_split = profile_model.split('.')
    if len(app_split) == 2:
        app_label, mod_label = app_split
    else:
        app_label, mod_label = app_split[-2:]
    pklss = models.get_model(app_label, mod_label)

    webiduser = WebIDUser.objects.get(id=user.id)
    try:
        pklss.objects.create(
            user=webiduser,
            first_name=" ",
            last_name=" ")
    except:
        raise


def custom_webid_uri(webiduser):
    return 'testing/webid/%s' % webiduser.username


# This function below copied and slightly modified from
# the builtin user creation method on WebIDAuth backend.

# If you want to access other fields not retrieved, you
# can alwas get the info from the raw profile stored in webidinfo.

def build_coop_user(request):
    """
    create a valid (and stored) django user to be associated with
    the authenticated WebID URI. This method will be used if no alternative
    is provided in the settings.WEBIDAUTH_CREATE_USER_CALLBACK value, or if
    the value there is not a valid function.
    """
    import re
    logger.debug('building user!')
    data = request.webidinfo
    validatedURI = data.validatedURI
    logger.debug('validatedURI = %s' % validatedURI)
    if not validatedURI:
        #XXX this check should be moved to "create_user" function
        logger.error('attempt to build an user with no validatedURI! \
skipping...')
        return None

    names = data.webid_name

    ###################################
    # XXX THIS IS THE COOP SPECIFIC BIT
    # XXX should take care of missing info.

    givenName = names.get('givenName', None)
    familyName = names.get('familyName', None)
    target_name = "%s.%s" % (givenName, familyName)

    ###################################

    def augment_name(name):
        ends_number = re.findall('\S+_(\d+)', name)
        if ends_number:
            number = int(ends_number[0]) + 1
            logger.debug('number %s' % number)
            _name = name.replace('_%s' % ends_number[0], '_%s' %
                    str(number))
            return _name
        else:
            return u"%s_1" % (name)

    #[warning] This is "a bit" hackish:
    max_tries = 20
    tries = max_tries

    while tries > 0:
        print 'looping'
        print 'target_name', target_name
        print 'try', max_tries - tries
        #XXX build kwargs dict, instead of hardcoding name
        colliding_users = models.WebIDUser.objects.all().\
                filter(username=target_name)
        #print 'colliding ', colliding_users
        #print colliding_users.count()
        #print 'all ', WebIDUser.objects.all()

        if colliding_users.count() > 0:
            target_name = augment_name(target_name)
            tries -= 1
        else:
            break
    else:
        logger.warning('Sorry... *that* name is already taken...')
        #XXX here we should signal some way of getting
        #user input (a form, or something)

    # XXX this assumption is VERY VERY weak
    # i.e, make sure to find a proper username if we do not
    # have the family name / given name
    # Also, handle collisions
    # (better to have, for instance, user@site usernames in the db
    # and have a display-name?)
    user = models.WebIDUser.objects.create(username=target_name)
    useruri = models.WebIDURI.objects.create(uri=validatedURI, user=user)
    useruri.save()
    logger.info('username set to %s' % names['name'])
    user.password = UserManager().make_random_password()
    user.is_active = True
    logger.debug('saving WebIDUser with name %s' % target_name)
    user.save()
    return user



# -*- coding:utf-8 -*-
from django.contrib import admin
from coop.org.models import BaseEngagement, BaseRelation
from coop.person.models import BasePerson
#from coop.exchange.models import BaseExchange, BasePaymentModality, BaseTransaction, BaseProduct
from django import forms
from django.conf import settings

# from django.contrib.admin.widgets import FilteredSelectMultiple
from django.db.models.loading import get_model
from django.utils.translation import ugettext_lazy as _
from coop.utils.autocomplete_admin import FkAutocompleteAdmin, InlineAutocompleteAdmin

from coop_local.models import Contact, Person
from coop_geo.models import Location
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

from django.contrib.admin.widgets import AdminURLFieldWidget
from django.db.models import URLField
from django.utils.safestring import mark_safe

from sorl.thumbnail.admin import AdminImageMixin
from tinymce.widgets import AdminTinyMCE

from chosen import widgets as chosenwidgets

if "coop.exchange" in settings.INSTALLED_APPS:
    from coop.exchange.admin import ExchangeInline

if "coop_geo" in settings.INSTALLED_APPS:
    from coop_geo.admin import LocatedInline, AreaInline

from django.contrib.contenttypes.generic import GenericTabularInline


class URLFieldWidget(AdminURLFieldWidget):
    def render(self, name, value, attrs=None):
        widget = super(URLFieldWidget, self).render(name, value, attrs)
        return mark_safe(u'%s&nbsp;&nbsp;<a href="#" onclick="window.'
                         u'open(document.getElementById(\'%s\')'
                         u'.value);return false;" class="btn btn-mini"/>Afficher dans une nouvelle fenêtre</a>' % (widget, attrs['id']))


class ContactInline(GenericTabularInline):
    model = get_model('coop_local', 'Contact')
    verbose_name = _(u'Contact information')
    verbose_name_plural = _(u'Contact informations')
    extra = 1


class EngagementInline(InlineAutocompleteAdmin):
    model = get_model('coop_local', 'Engagement')
    verbose_name = _(u'Member')
    verbose_name_plural = _(u'Members')
    related_search_fields = {'person': ('last_name', 'first_name',
                                        'email', 'structure', 'username'), }
    extra = 2


class RelationInline(InlineAutocompleteAdmin):
    model = get_model('coop_local', 'Relation')
    fk_name = 'source'
    readonly_fields = ('created',)
    fields = ('reltype', 'target', 'created')
    related_search_fields = {'target': ('title', 'subtitle', 'description'), }
    extra = 1


class OrgInline(InlineAutocompleteAdmin):
    model = get_model('coop_local', 'Engagement')
    verbose_name = _(u'Engagement')
    verbose_name_plural = _(u'Engagements')
    related_search_fields = {'organization': ('title', 'subtitle', 'description'), }
    extra = 1


class OrganizationAdminForm(forms.ModelForm):
    description = forms.CharField(widget=AdminTinyMCE(attrs={'cols': 80, 'rows': 60}), required=False)

    class Meta:
        model = get_model('coop_local', 'Organization')
        widgets = {'category': chosenwidgets.ChosenSelectMultiple()}

    def __init__(self, *args, **kwargs):
        super(OrganizationAdminForm, self).__init__(*args, **kwargs)
        engagements = self.instance.engagement_set.all()
        members_id = engagements.values_list('person_id', flat=True)
        org_contacts = Contact.objects.filter(
            Q(content_type=ContentType.objects.get(model='organization'), object_id=self.instance.id)
          | Q(content_type=ContentType.objects.get(model='person'), object_id__in=members_id)
            )
        self.fields['pref_email'].queryset = org_contacts.filter(category=8)
        self.fields['pref_phone'].queryset = org_contacts.filter(category=1)
        self.fields['category'].help_text = None

        member_locations_id = [m.location.id for m in
            Person.objects.filter(id__in=members_id).exclude(location=None)]  # limit SQL to location field

        self.fields['pref_address'].queryset = Location.objects.filter(
            Q(id__in=self.instance.located.all().values_list('location_id', flat=True))
          | Q(id__in=member_locations_id)
            )


def create_action(category):
    def add_cat(modeladmin, request, queryset):
        for obj in queryset:
            obj.category.add(category)
    name = "cat_%s" % (category.slug,)
    return (name, (add_cat, name, _(u'Add to the "%s" category') % (category,)))


class OrganizationAdmin(AdminImageMixin, FkAutocompleteAdmin):
    change_form_template = 'admintools_bootstrap/tabbed_change_form.html'
    form = OrganizationAdminForm
    list_display = ['logo_list_display', 'label', 'active', 'has_description', 'has_location']
    list_display_links = ['label', ]
    search_fields = ['title', 'acronym', 'subtitle', 'description']
    list_filter = ['active', 'category']
    #actions_on_top = True
    #actions_on_bottom = True
    #save_on_top = True
    #filter_horizontal = ('category',)
    list_per_page = 10
    list_select_related = True
    #read_only_fields = ['created','modified']
    ordering = ('title',)
    formfield_overrides = {
        URLField: {'widget': URLFieldWidget},
    }

    if "coop.exchange" in settings.INSTALLED_APPS:
        inlines = [ ContactInline,
                        EngagementInline,
                        ExchangeInline,
                        RelationInline,
                        LocatedInline,
                        AreaInline,
                        ]
    else:
        inlines = [ ContactInline,
                    EngagementInline,
                    LocatedInline,
                    ]

    # grace au patch
    # https://code.djangoproject.com/ticket/17856
    # https://github.com/django/django/blob/master/django/contrib/admin/options.py#L346
    # def get_inline_instances(self, request, obj):
    #     inline_instances = []

    #     for inline_class in self.inlines:
    #         if inline_class.model == get_model('coop_local', 'Exchange'):
    #             inline = inline_class(self.model, self.admin_site, obj=obj)
    #         else:
    #             inline = inline_class(self.model, self.admin_site)
    #         inline_instances.append(inline)
    #     return inline_instances

    fieldsets = (
        ('Identité', {
            'fields': ('logo', 'title', ('acronym', 'pref_label'), 'subtitle', ('birth', 'active',),
                        'web')
            }),
        ('Description', {
            'fields': ('description', 'category', 'tags', )
            }),

        ('Préférences', {
            #'classes': ('collapse',),
            'fields': ('pref_email', 'pref_phone', 'pref_address', 'notes',)
        })
    )

    def get_actions(self, request):
        myactions = dict(create_action(s) for s in get_model('coop_local', 'OrganizationCategory').objects.all())
        return dict(myactions, **super(OrganizationAdmin, self).get_actions(request))  # merging two dicts
        #list_display = ['my_image_thumb', 'my_other_field1', 'my_other_field2', ] ???

    def get_form(self, request, obj=None, **kwargs):
        # just save obj reference for future processing in Inline
        request._obj_ = obj
        return super(OrganizationAdmin, self).get_form(request, obj, **kwargs)

    class Media:
        js = ('/static/js/admin_customize.js',)




# -*- coding:utf-8 -*-
from django.contrib import admin
from django.contrib.admin.options import InlineModelAdmin
from django.conf import settings
from tinymce.widgets import AdminTinyMCE
from django import forms
from sorl.thumbnail.admin import AdminImageMixin
from django.utils.translation import ugettext_lazy as _


class ObjEnabledInline(InlineModelAdmin):
    # def __init__(self, *args, **kwargs):
    #     self.parent_object = kwargs['obj']
    #     del kwargs['obj']  # superclass will choke on this
    #     super(ObjEnabledInline, self).__init__(*args, **kwargs)

    def get_formset(self, request, obj=None, **kwargs):
        # Hack! Hook parent obj just in time to use in formfield_for_manytomany
        self.parent_obj = obj
        return super(ObjEnabledInline, self).get_formset(
            request, obj, **kwargs)


class ObjEnabledAdmin(admin.ModelAdmin):  # TODO bring back the permissions code from original method ?

    def get_form(self, request, obj=None, **kwargs):
        # Hack! Hook parent obj just in time to use in formfield_for_manytomany
        self._obj = obj
        return super(ObjEnabledAdmin, self).get_form(
            request, obj, **kwargs)


if "coop_cms" in settings.INSTALLED_APPS:

    from coop_cms.admin import NavTreeAdmin, ArticleAdmin
    from coop_cms.models import NavTree
    from coop_cms.settings import get_article_class
    from coop_cms.forms import ArticleAdminForm

    # -- We need to customize coop-cms NavtreeAdmin

    class MyNavTreeAdmin(NavTreeAdmin):
        change_form_template = 'admin/navtree_change_form.html'

    admin.site.unregister(NavTree)
    admin.site.register(NavTree, MyNavTreeAdmin)

    # -- We need to customize coop-cms ArticleAdmin

    class CoopArticleForm(ArticleAdminForm):
        content = forms.CharField(widget=AdminTinyMCE(attrs={'cols': 80, 'rows': 60}), required=False)

    class CoopArticleAdmin(ArticleAdmin, AdminImageMixin):
        form = CoopArticleForm
        change_form_template = 'admintools_bootstrap/tabbed_change_form.html'
        change_list_template = 'admin/article_change_list.html'
        list_filter = ['category']

        list_display = ['logo_list_display', 'title', 'publication', 'headline', 'in_newsletter', 'category']
        list_editable = ['publication', 'in_newsletter', 'headline', 'category']
        #list_display = ['logo_list_display', 'title', 'publication', 'category', 'modified', 'in_newsletter']
        #list_editable = ['publication', 'in_newsletter', 'category']
        list_display_links = ['title']

        readonly_fields = ['created', 'modified']
        fieldsets = (
            #(_('Navigation'), {'fields': ('navigation_parent',)}),
            ('Edition', {'fields': ('title', 'logo', 'content', 'template')}),
            ('Options', {'fields': ('summary', 'category', 'is_homepage', 'in_newsletter')}),
        )

        def get_form(self, request, obj=None, **kwargs):
            form = super(ArticleAdmin, self).get_form(request, obj, **kwargs)
            form.current_user = request.user
            return form

    admin.site.unregister(get_article_class())
    admin.site.register(get_article_class(), CoopArticleAdmin)


if 'forms_builder.forms' in settings.INSTALLED_APPS:

    from forms_builder.forms.admin import FormAdmin
    from forms_builder.forms.models import Form

    class MyFormAdmin(FormAdmin):
        change_form_template = 'admintools_bootstrap/tabbed_change_form.html'
        fieldsets = [
            (_("Settings"), {"fields": ( "title",
                                    #("status", "login_required",),
                                    #("publish_date", "expiry_date",),
                                     "intro", "button_text", "response")}),
            (_("Email"), {"fields": ("send_email", "email_from", "email_copies",
                                     "email_subject", "email_message")}),
            ]
        list_display = ("title", "total_entries", "admin_links")
        list_display_links = ("title",)
        list_editable = []
    admin.site.unregister(Form)
    admin.site.register(Form, MyFormAdmin)



#     def get_inline_instances(self, request, obj):
#         inline_instances = []
#         for inline_class in self.inlines:
#             if inline_class.__mro__.__contains__(ObjEnabledInline):
#                 inline = inline_class(self.model, self.admin_site, obj=obj)
#             else:
#                 inline = inline_class(self.model, self.admin_site)
#             inline_instances.append(inline)
#         return inline_instances



# from coop.org.models import BaseEngagement, BaseRelation
# from coop.person.models import BasePerson
# from coop.exchange.models import BaseExchange, BasePaymentModality, BaseTransaction, BaseProduct
# from django import forms
# from django.conf import settings

# # from django.contrib.admin.widgets import FilteredSelectMultiple
# from django.db.models.loading import get_model
# from django.utils.translation import ugettext_lazy as _
# from coop.utils.autocomplete_admin import FkAutocompleteAdmin, InlineAutocompleteAdmin

# from coop_local.models import Contact, Person
# from coop_geo.models import Location
# from django.contrib.contenttypes.models import ContentType
# from django.db.models import Q

# from django.contrib.admin.widgets import AdminURLFieldWidget
# from django.db.models import URLField
# from django.utils.safestring import mark_safe

# from sorl.thumbnail.admin import AdminImageMixin

# from sorl.thumbnail import default
# ADMIN_THUMBS_SIZE = '60x60'


# class SimpleLinkWidget(forms.Widget):
#     def __init__(self, obj, attrs=None):
#         self.object = obj
#         super(SimpleLinkWidget, self).__init__(attrs)

#     def render(self, name, value, attrs=None):
#         if self.object.pk:
#             return mark_safe(
#                 u'<a target="_blank" href="../../../%s/%s/%s/">%s</a>' %\
#                       (
#                        self.object._meta.app_label,
#                        self.object._meta.object_name.lower(),
#                        self.object.pk,
#                        u'<b>Compléter la fiche détaillée</b>'
#                        )
#             )
#         else:
#             return mark_safe(u'')


# class M2MLinkWidget(forms.Widget):
#     def __init__(self, obj, fkey_name, attrs=None,):
#         self.object = obj
#         self.fkey = fkey_name
#         super(M2MLinkWidget, self).__init__(attrs)

#     def render(self, name, value, attrs=None):
#         if self.object.pk and self.fkey and getattr(self.object, self.fkey) != None:
#             return mark_safe(u'<a href="../../../%s/%s/%s/">%s</a>' % \
#                 (self.object._meta.app_label,
#                     getattr(self.object, self.fkey)._meta.object_name.lower(),
#                     getattr(self.object, self.fkey).pk,
#                     u' '.join((u'Fiche', unicode(self.fkey)))  # TODO how to get the translated app name ?
#                     )
#             )
#         else:
#             return mark_safe(u'')


# class ExchangeInlineLinkForm(forms.ModelForm):
#     class Meta:
#         model = BaseExchange
#     lien = forms.CharField(label='lien', required=False)

#     def __init__(self, *args, **kwargs):
#         super(ExchangeInlineLinkForm, self).__init__(*args, **kwargs)
#         self.fields['lien'].widget = SimpleLinkWidget(self.instance)


# class ContactInlineLinkForm(forms.ModelForm):
#     lien = forms.CharField(label='lien', required=False)

#     def __init__(self, *args, **kwargs):
#         super(ContactInlineLinkForm, self).__init__(*args, **kwargs)
#         self.fields['lien'].widget = M2MLinkWidget(self.instance, fkey_name='person')


# class OrgInlineLinkForm(forms.ModelForm):
#     lien = forms.CharField(label='lien', required=False)

#     def __init__(self, *args, **kwargs):
#         super(OrgInlineLinkForm, self).__init__(*args, **kwargs)
#         self.fields['lien'].widget = M2MLinkWidget(self.instance, fkey_name='organization')


# # ---------- if coop-geo is installed --------------

# if "coop_geo" in settings.INSTALLED_APPS:
#     from coop_geo.admin import LocatedInline, AreaInline

# # ----------------------------------------------------








# # from django.contrib.admin.filterspecs import FilterSpec, RelatedFilterSpec
# # from django.contrib.admin.util import get_model_from_relation
# # from django.db.models import Count


# # if "coop_tag" in settings.INSTALLED_APPS:

# #     #from taggit.managers import TaggableManager
# #     from taggit_autosuggest.managers import TaggableManager

# #     class TaggitFilterSpec(RelatedFilterSpec):
# #         """
# #         A FilterSpec that can be used to filter by taggit tags in the admin.
# #         To use, simply import this module (for example in `models.py`), and add the
# #         name of your :class:`taggit.managers.TaggableManager` field in the
# #         :attr:`list_filter` attribute of your :class:`django.contrib.ModelAdmin`
# #         class.
# #         """

# #         def __init__(self, f, request, params, model, model_admin,
# #                      field_path=None):
# #             super(RelatedFilterSpec, self).__init__(
# #                 f, request, params, model, model_admin, field_path=field_path)

# #             other_model = get_model_from_relation(f)
# #             if isinstance(f, (models.ManyToManyField,
# #                               models.related.RelatedObject)):
# #                 # no direct field on this model, get name from other model
# #                 self.lookup_title = other_model._meta.verbose_name
# #             else:
# #                 self.lookup_title = f.verbose_name  # use field name
# #             rel_name = other_model._meta.pk.name
# #             self.lookup_kwarg = '%s__%s__exact' % (self.field_path, rel_name)
# #             self.lookup_kwarg_isnull = '%s__isnull' % (self.field_path)
# #             self.lookup_val = request.GET.get(self.lookup_kwarg, None)
# #             self.lookup_val_isnull = request.GET.get(
# #                                           self.lookup_kwarg_isnull, None)
# #             # Get tags and their count
# #             # through_opts = f.through._meta
# #             # count_field = ("%s_%s_items" % (through_opts.app_label,
# #             #         through_opts.object_name)).lower()

# #             queryset = getattr(f.model, f.name).all()
# #             queryset = queryset.annotate(num_times=Count('ctagged_items'))
# #             queryset = queryset.order_by("-num_times")
# #             self.lookup_choices = [(t.pk, "%s (%s)" % (t.name, t.num_times))
# #                     for t in queryset[:20]]


# #     # HACK: we insert the filter at the beginning of the list to avoid the manager
# #     # to be picked as a RelatedFilterSpec
# #     FilterSpec.filter_specs.insert(0, (lambda f: isinstance(f, TaggableManager),
# #         TaggitFilterSpec))



# -*- coding:utf-8 -*-
from django.conf.urls.defaults import *
from django.views.generic.detail import DetailView
from django.views.generic import ListView

from coop_local.models import Exchange

# urlpatterns = patterns('',
#     url(r'^(?P<uuid>[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})/$', 
#         'coop.exchange.views.exchange_detail', 
#         name="exchange_detail"),
# )

urlpatterns = patterns('',
    url(r'^$', ListView.as_view(model=Exchange, template_name="exchange/exchange_list.html")),
    url(r'^(?P<slug>\d+)/$', 
        DetailView.as_view(
            model=Exchange,
            context_object_name="exchange",
            slug_field="id",
            template_name="exchange/exchange_detail.html"
            ),
            name="exchange_detail"),
    )



# -*- coding:utf-8 -*-
from django.contrib import admin
from django import forms
from coop.exchange.models import BaseTransaction, BaseProduct
from django.db.models.loading import get_model
from django.utils.translation import ugettext_lazy as _
from coop.utils.autocomplete_admin import FkAutocompleteAdmin, InlineAutocompleteAdmin
from coop_geo.admin import LocatedInline
from tinymce.widgets import AdminTinyMCE
from coop.utils.fields import MultiSelectFormField, MethodsCheckboxSelectMultiple, DomainCheckboxSelectMultiple
from coop.exchange.models import ETYPE
from django.conf import settings

from coop.admin import ObjEnabledInline

# class PaymentInline(admin.TabularInline):
#     model = get_model('coop_local', 'PaymentModality')
#     #model = BasePaymentModality
#     extra = 0

if 'coop.exchange' in settings.INSTALLED_APPS:

    class ExchangeMethodForm(forms.ModelForm):
        etypes = MultiSelectFormField(widget=DomainCheckboxSelectMultiple(), choices=ETYPE.CHOICES)

        class Meta:
            model = get_model('coop_local', 'ExchangeMethod')

    class ExchangeMethodAdmin(admin.ModelAdmin):  # AdminImageMixin,
        form = ExchangeMethodForm
        list_display = ('label', 'applications')

    class ExchangeForm(forms.ModelForm):
        description = forms.CharField(widget=AdminTinyMCE(attrs={'cols': 80, 'rows': 60}), required=False)
        methods = forms.ModelMultipleChoiceField(   queryset=get_model('coop_local', 'ExchangeMethod').objects.all(),
                                                    widget=MethodsCheckboxSelectMultiple(),
                                                    required=False)

        def __init__(self, *args, **kwargs):
            super(ExchangeForm, self).__init__(*args, **kwargs)
        #   self.fields['methods'].widget = forms.CheckboxSelectMultiple()
            self.fields['methods'].help_text = ''
            self.fields['methods'].label = _(u'exchange methods')
            self.fields['tags'].label = 'Tags'

        class Media:
            js = ('js/select_exchange_methods.js',)

        class Meta:
            model = get_model('coop_local', 'Exchange')

    class ExchangeInline(admin.StackedInline, ObjEnabledInline):
        form = ExchangeForm
        model = get_model('coop_local', 'Exchange')
        fieldsets = ((None, {'fields': (('eway', 'etype'),
                                         'methods',
                                         'title',
                                        'description', 'tags',
                                        'location', 'area')
                            }),)
        extra = 1

        def formfield_for_dbfield(self, db_field, **kwargs):
            if self.parent_obj != None:
                if db_field.name == 'location':
                    kwargs['queryset'] = self.parent_obj.locations()  # default : only org's locations
                if db_field.name == 'area':
                    kwargs['queryset'] = self.parent_obj.areas()  # default : only org's areas
                if db_field.name == 'person':
                    kwargs['queryset'] = self.parent_obj.members.all()  # default : only org's members
            return super(ExchangeInline, self).formfield_for_dbfield(db_field, **kwargs)

    class ProductInline(admin.StackedInline):
        model = BaseProduct
        fieldsets = ((None, {'fields': ('title',
                                        'description',
                                        )}),)
        extra = 1

    class TransactionInline(admin.StackedInline):
        model = BaseTransaction
        fk_name = 'destination_org'
        fieldsets = ((None, {'fields': ('title', 'origin', 'description')}),)
        #related_search_fields = {'origin': ('title', 'description', 'organization__title'), }
        extra = 1

    class ExchangeAdmin(FkAutocompleteAdmin):  # AdminImageMixin,
        form = ExchangeForm
        list_display = ('title', 'etype')  # , 'methods')
        # TODO to be finished ...
        # list_editable = ('methods',)
        related_search_fields = {'organization': ('title', 'subtitle', 'description'), }
        fieldsets = ((None, {'fields': (('eway', 'etype'),
                                         'methods',
                                         'title',
                                         'organization',
                                        'description', 'tags')
                            }),)
        inlines = [
                #PaymentInline,
                LocatedInline,  # Using coop-geo
            ]



# -*- coding:utf-8 -*-
# -*- coding:utf-8 -*-
from django.conf.urls.defaults import patterns, include, url
from django.views.generic.base import TemplateView, RedirectView
from django.conf import settings
from coop.feeds import UpdateFeed


class TextPlainView(TemplateView):
    def render_to_response(self, context, **kwargs):
        return super(TextPlainView, self).render_to_response(
              context, content_type='text/plain', **kwargs)

urlpatterns = patterns('',

    url(r'^$', 'coop.views.home', name="home"),

    (r'^org/', include('coop.org.urls')),
    (r'^comments/', include('django.contrib.comments.urls')),

    # a revoir
    (r'^perso/$', 'coop.person.views.perso'),
    (r'^membre/', include('coop.person.urls')),

    url(r'^feed/(?P<model>[\w-]+)/$', UpdateFeed()),
    url(r'^hub/', include('subhub.urls')),

    url(r'^robots\.txt$', TextPlainView.as_view(template_name='robots.txt')),
    url(r'^favicon\.ico$', RedirectView.as_view(url='/static/img/favicon.ico')),
    url(r'^d2r/(?P<mode>[\w-]+)/mapping.ttl$', 'coop.views.d2r_mapping'),

)

if 'coop.exchange' in settings.INSTALLED_APPS:

    urlpatterns = urlpatterns + patterns('',
        (r'^annonces/', include('coop.exchange.urls')),
    )




# -*- coding:utf-8 -*-
from django.db import models
from extended_choices import Choices
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.conf import settings
from coop.models import URIModel
from sorl.thumbnail import ImageField
from sorl.thumbnail import default
import rdflib
import logging
ADMIN_THUMBS_SIZE = '60x60'


DISPLAY = Choices(
    ('PUBLIC',  1,  _(u'public information')),
    ('USERS',   2,  _(u'registered users')),
    ('MEMBER',  3,  _(u'members of an organization')),
    ('RELATED', 4,  _(u'related organizations members')),
    ('ORG',     5,  _(u'members of this organization')),
    ('ADMIN',   6,  _(u'administrators of this site')),
)

# will apply to contact numbers and other things
# TODO simplify it, see PPO Ontology

COMM_MEANS = Choices(
    ('MAIL',    8,  _(u'E-mail')),
    ('LAND',    1,  _(u'Landline phone')),
    ('GSM',     2,  _(u'Mobile phone')),
    ('FAX',     3,  _(u'Fax')),
    ('WEB',     9,  _(u'Secondary web site')),
    ('SKYPE',   4,  _(u'Skype')),
    ('TWITTER', 5,  _(u'Twitter')),
    ('RSS',     6,  _(u'RSS Feed')),
    ('VCAL',    7,  _(u'vCalendar')),
)


# A model to deal with how contact (an organization, persone,...)
# mail,fax, ... see COMM_MEANS
class BaseContact(URIModel):
    category = models.PositiveSmallIntegerField(_(u'Category'),
                    choices=COMM_MEANS.CHOICES)
    content = models.CharField(_(u'content'), max_length=250)
    details = models.CharField(_(u'details'), blank=True, max_length=100)
    display = models.PositiveSmallIntegerField(_(u'Display'),
                    choices=DISPLAY.CHOICES, default=DISPLAY.PUBLIC)
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    class Meta:
        abstract = True
        ordering = ['category']
        verbose_name = _(u'Contact')
        verbose_name_plural = _(u'Contacts')

    def __unicode__(self):
        if self.content_object != None:
            return self.content + u' (' + self.content_object.__unicode__() + u')'
        else:
            return self.content

    def label(self):
        return self.__unicode__()

    def clean(self):
        if self.category in [1, 2, 3]:
            import re
            phoneSplitRegex = re.compile(r"[\-\(\) \.\,]")
            parts = phoneSplitRegex.split(self.content)
            num = ''.join(parts)
            if len(num) == 10:
                self.content = '.'.join((num[:2], num[2:4], num[4:6], num[6:8], num[8:]))
            else:
                raise ValidationError(_(u'Phone numbers must have 10 digits'))

    def save(self, *args, **kwargs):
        self.clean()
        #logging.error(u'A contact has been created or modified', exc_info=True, extra={'request': request})
        super(BaseContact, self).save(*args, **kwargs)


class BaseRole(URIModel):
    label = models.CharField(_(u'label'), max_length=60)
    slug = exfields.AutoSlugField(populate_from=('label'), overwrite=True)

    domain_name = 'data.economie-solidaire.fr'

    @property
    def uri_id(self):
        return self.slug

    class Meta:
        abstract = True
        ordering = ['label']

    def __unicode__(self):
        return unicode(self.label)

    def get_absolute_url(self):
        return reverse('role_detail', args=[self.slug])


RELATIONS = Choices(
    ('MEMBER',          1,  _(u' is member of ')),
    ('REG_SUPPLIER',    2,  _(u' has for regular supplier ')),
    ('OCC_SUPPLIER',    3,  _(u' has for casual supplier ')),
    ('SUPPORT',         4,  _(u' received technical support from ')),
    ('FUNDING',         5,  _(u' received financial support from ')),
)


class BaseRelation(models.Model):
    source = models.ForeignKey('coop_local.Organization', verbose_name=_(u'source organization'), related_name='source')
    target = models.ForeignKey('coop_local.Organization', verbose_name=_(u'target organization'), related_name='target')
    reltype = models.PositiveSmallIntegerField(_(u'Relation type'), choices=RELATIONS.CHOICES)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    #confirmed = models.BooleanField(default=False, verbose_name=_(u'confirmed by the target organization'))

    class Meta:
        abstract = True

    def __unicode__(self):
        return u'"' + self.source.__unicode__() + u'"' + \
            unicode(RELATIONS.CHOICES_DICT[self.reltype]) + \
            u'"' + self.target.__unicode__() + u'".'
    '''
    def save(self):
        vérifier si la relation inverse existe
    '''


class BaseEngagement(URIModel):
    person = models.ForeignKey('coop_local.Person', verbose_name=_(u'person'), related_name='engagements')
    organization = models.ForeignKey('coop_local.Organization', verbose_name=_(u'organization'))
    role = models.ForeignKey('coop_local.Role', verbose_name=_(u'role'))
    role_detail = models.CharField(_(u'detailed role'), blank=True, max_length=100)
    org_admin = models.BooleanField(_(u'has editor rights'), default=True)
    engagement_display = models.PositiveSmallIntegerField(_(u'Display'), choices=DISPLAY.CHOICES, default=DISPLAY.PUBLIC)

    class Meta:
        abstract = True
        verbose_name = _('Engagement')
        verbose_name_plural = _('Engagements')

    def __unicode__(self):
        return '%(person)s, %(role)s @ %(org)s' % {
                    'person': self.person.__unicode__(),
                    'role': self.role.__unicode__(),
                    'org': self.organization.__unicode__()
                    }

    def label(self):
        return self.__unicode__()


class BaseOrganizationCategory(models.Model):
    label = models.CharField(blank=True, max_length=100)
    slug = exfields.AutoSlugField(populate_from=('label'), overwrite=True)

    class Meta:
        abstract = True
        verbose_name = _(u'organization category')
        verbose_name_plural = _(u'organization categories')

    def __unicode__(self):
        return self.label

    #@models.permalink
    def get_absolute_url(self):
        return reverse('org_category_detail', args=[self.slug])



PREFLABEL = Choices(
    ('TITLE',   1,  _(u'title')),
    ('ACRO',    2,  _(u'acronym')),
)


class BaseOrganization(URIModel):
    title = models.CharField(_(u'title'), max_length=250)

    acronym = models.CharField(_(u'acronym'), max_length=20, blank=True, null=True)
    pref_label = models.PositiveSmallIntegerField(_(u'Preferred label'),
                        choices=PREFLABEL.CHOICES, default=PREFLABEL.TITLE)

    subtitle = models.CharField(_(u'tagline'), blank=True, null=True,
                max_length=250,
                help_text=_(u'tell us what your organization do in one line.'))

    description = models.TextField(_(u'description'), blank=True, null=True)

    logo = ImageField(upload_to='logos/', null=True, blank=True)

    relations = models.ManyToManyField('coop_local.Organization',
                symmetrical=False, through='coop_local.Relation',
                verbose_name=_(u'relations'))

    category = models.ManyToManyField('coop_local.OrganizationCategory',
                blank=True, null=True, verbose_name=_(u'category'))

    members = models.ManyToManyField('coop_local.Person',
                through='coop_local.Engagement', verbose_name=_(u'members'))

    contacts = generic.GenericRelation('coop_local.Contact')
    subs = generic.GenericRelation('coop_local.Subscription')

    # coop_geo must be loaded BEFORE coop_local
    if "coop_geo" in settings.INSTALLED_APPS:
        located = generic.GenericRelation('coop_geo.Located')
        framed = generic.GenericRelation('coop_geo.AreaLink')

    birth = models.DateField(_(u'creation date'), null=True, blank=True)
    email = models.EmailField(_(u'global email'), blank=True, null=True)
    email_sha1 = models.CharField(_(u'email checksum'),
            max_length=250, blank=True, null=True)  # TODO : do this in Postgre
    web = models.URLField(_(u'web site'), blank=True, null=True)

    pref_email = models.ForeignKey('coop_local.Contact',
                verbose_name=_(u'preferred email'),
                related_name="pref_email", null=True, blank=True)
    pref_phone = models.ForeignKey('coop_local.Contact',
                verbose_name=_(u'preferred phone'),
                related_name='pref_phone', null=True, blank=True)
    pref_address = models.ForeignKey('coop_geo.Location',
                verbose_name=_(u'preferred postal address'),
                related_name='pref_adress', null=True, blank=True)

    slug = exfields.AutoSlugField(populate_from='title', blank=True, overwrite=True)
    active = models.BooleanField(_(u'show on public site'), default=True,)
    notes = models.TextField(_(u'notes'), blank=True)

    class Meta:
        abstract = True
        ordering = ['title']
        verbose_name = _(u'Organization')
        verbose_name_plural = _(u'Organizations')

    def __unicode__(self):
        return unicode(self.title)

    def label(self):
        if self.pref_label == PREFLABEL.TITLE:
            return self.title
        elif self.pref_label == PREFLABEL.ACRO:
            return self.acronym

    def can_edit_organization(self, user):
        return True

    if "coop_geo" in settings.INSTALLED_APPS:

        def has_location(self):
            return self.located.all().count() > 0
        has_location.boolean = True
        has_location.short_description = _(u'geo')

        def locations(self):
            from coop_geo.models import Location
            return Location.objects.filter(id__in=self.located.all().values_list('location_id', flat=True))

        def areas(self):
            from coop_geo.models import Area
            return Area.objects.filter(id__in=self.framed.all().values_list('location_id', flat=True))

    def has_description(self):
        return self.description != None and len(self.description) > 20
    has_description.boolean = True
    has_description.short_description = _(u'desc.')

    def logo_list_display(self):
        if self.logo:
            thumb = default.backend.get_thumbnail(self.logo.file, ADMIN_THUMBS_SIZE)
            return '<img width="%s" src="%s" />' % (thumb.width, thumb.url)
        else:
            return _(u"No Image")
    logo_list_display.short_description = _(u"logo")
    logo_list_display.allow_tags = True

    #@models.permalink
    def get_absolute_url(self):
        return reverse('org_detail', args=[self.slug])

    def get_tags(self):
        return self.tags.all()

    def get_relations(self):
        relations = {}
        relmap = RELATIONS.REVERTED_CHOICES_CONST_DICT

        for rel in self.source.all():
            reltype = str('OUT_' + relmap[rel.reltype])  # me => others
            relations[reltype] = []
            relations[reltype].append(rel.target)
        for rel in self.target.all():
            reltype = str('IN_' + relmap[rel.reltype])  # others said this
            if reltype not in relations:
                relations[reltype] = []
            #if rel.confirmed:  # which one are confirmed by both parts
            relations[reltype].append(rel.source)
        return relations

    # def local_uri(self):
    #     return ('http://dev.credis.org:8000/org/' + self.slug + '/')

    def main_location(self):
        return self.located.all()[0].location

    def save(self, *args, **kwargs):
        # Set default values for preferred email, phone and postal address
        if self.pref_phone == None:
            fixes = self.contacts.filter(category=1)
            if fixes.count() == 1:
                self.pref_phone = fixes[0]
        if self.pref_email == None:
            orgmails = self.contacts.filter(category=8)
            if orgmails.count() == 1:
                self.pref_email = orgmails[0]
        if 'coop_geo' in settings.INSTALLED_APPS:
            if self.pref_address == None:
                locations = self.located.all()  # should we have a "main venue" ?
                if locations.count() == 1:
                    self.pref_address = locations[0].location

        # TODO move this to Contact model or do it in SQL

        # if self.email and self.email != '':
        #     import hashlib
        #     m = hashlib.sha1()
        #     m.update(self.email)
        #     self.email_sha1 = m.hexdigest()
        super(BaseOrganization, self).save(*args, **kwargs)


    # title field needs a special handling. checkDirectMap does not work
    # because two rdf property use the coop_local_organization.title field
    # Thus we have to decide which one to use
    def updateField_title(self, dbfieldname, graph):
            title = list(graph.objects(rdflib.term.URIRef(self.uri), settings.NS_LEGAL.legalName))
            if len(title) == 1:
                self.title = title[0]
                print "For id %s update the field %s" % (self.id, dbfieldname)
            elif len(title) == 0:
                title = list(graph.objects(rdflib.term.URIRef(self.uri), settings.NS_RDFS.label))
                if len(title) == 1:
                    self.title = title[0]
                    print "For id %s update the field %s" % (self.id, dbfieldname)
                else:
                    print "    The field %s cannot be updated." % dbfieldname
            else:
                print "    The field %s cannot be updated." % dbfieldname


    def updateField_pref_label(self, dbfieldname, graph):
        prefLabel = list(graph.objects(rdflib.term.URIRef(self.uri), settings.NS_SKOS.prefLabel))
        if len(prefLabel) == 1:
            legalName = list(graph.objects(rdflib.term.URIRef(self.uri), settings.NS_LEGAL.legalName))
            if len(legalName) == 1:
                if prefLabel[0] == legalName[0]:
                    self.pref_label = PREFLABEL.TITLE
                    print "For id %s update the field %s" % (self.id, dbfieldname)

            else:
                acronym = list(graph.objects(rdflib.term.URIRef(self.uri), settings.NS_OV.acronym))
                if len(acronym) == 1:
                    if prefLabel[0] == acronym[0]:
                        self.pref_label = PREFLABEL.ACRO
                        print "For id %s update the field %s" % (self.id, dbfieldname)

                    else:
                        print "    The field %s cannot be updated." % dbfieldname
                else:
                    print "    The field %s cannot be updated." % dbfieldname
        else:
            print "    The field %s cannot be updated." % dbfieldname







# -*- coding:utf-8 -*-
from coop.models import StaticURIModel
from django.db.models.signals import post_save
from django.dispatch import receiver
# from django.core.signals import  request_finished
# from coop_local.models import Organization
from django.contrib.sites.models import Site
from django.conf import  settings
import subhub
import logging
from redis.exceptions import ConnectionError


log = logging.getLogger('subhub.maintenance')

try:
    import django_rq
    redis_conn = django_rq.get_connection('high')
    q = django_rq.get_queue('high')
except:
    log.warning('Redis is not enabled')
    pass


# Workaround for rqworker limitation
def letsCallDistributionTaskProcess():
    subhub.models.DistributionTask.objects.process()


# Listener tool
@receiver(post_save)
def post_save_callback(sender, instance, **kwargs):
    log.debug(u"Post save callback with sender %s and instance %s" % (sender, instance))
    if StaticURIModel in sender.__mro__:
        feed_url = 'http://%s/feed/%s/' % (Site.objects.get_current().domain, sender.__name__.lower())
        try:
            subhub.publish([feed_url], instance.uri, False)
        except Exception, e:
            log.warning(u'Unable to publish %s for feed %s : %s' % (instance, feed_url, e))
        if settings.SUBHUB_MAINTENANCE_AUTO:
            try:
                q.enqueue(letsCallDistributionTaskProcess)
            except ConnectionError, e:
                log.warning(u"%s" % e)
    elif isinstance(instance, subhub.models.SubscriptionTask) and settings.SUBHUB_MAINTENANCE_AUTO:
        # call the maintenance
            try:
                log.info(u'Processing verification queue...')
                subhub.models.SubscriptionTask.objects.process(log=log)
            except subhub.utils.LockError, e:
                log.warning(u"%s" % e)
    else:
        pass



from rdflib import Namespace

DEFAULT_RDF_NAMESPACES = {
 'ess':  u'http://ns.economie-solidaire.fr/ess#',
 'dct': u'http://purl.org/dc/terms/',
 'org': u'http://www.w3.org/ns/org#',
 'ctag': u'http://commontag.org/ns#',
 'foaf': u'http://xmlns.com/foaf/0.1/',
 'locn': u'http://www.w3.org/ns/locn#',
 'skos': u'http://www.w3.org/2004/02/skos/core#',
 'legal': u'http://www.w3.org/ns/legal#',
 'vcard': u'http://www.w3.org/2006/vcard/ns#',
 'xsd': 'http://www.w3.org/2001/XMLSchema#',
 'geofr': u'http://rdf.insee.fr/geo/',
 'skosxl': u'http://www.w3.org/2008/05/skos-xl#',
 'gr': u'http://purl.org/goodrelations/v1#',
 'event': u'http://purl.org/NET/c4dm/event.owl#',
 'sioc': u'http://rdfs.org/sioc/ns#',
 'opens': u'http://rdf.opensahara.com/type/geo/',
 'person': u'http://www.w3.org/ns/person#',
 'schema': u'http://schema.org/',
 'rss': u'http://purl.org/net/rss1.1#',
 'ov': u'http://open.vocab.org/terms/'
 }


 # Useful stuffs
NS_D2RQ = Namespace('http://www.wiwiss.fu-berlin.de/suhl/bizer/D2RQ/0.1#')
NS_LEGAL = Namespace(DEFAULT_RDF_NAMESPACES['legal'])
NS_ESS = Namespace(DEFAULT_RDF_NAMESPACES['ess'])
NS_SKOS = Namespace(DEFAULT_RDF_NAMESPACES['skos'])
NS_OV = Namespace(DEFAULT_RDF_NAMESPACES['ov'])
NS_RDFS = Namespace(u'http://www.w3.org/2000/01/rdf-schema#')



from django import template
from urlparse import urlparse, urlunparse
from django.core.exceptions import FieldError
from django.db.models.fields import FieldDoesNotExist

register = template.Library()


@register.simple_tag
def favicon(url):
    parsed_url = urlparse(url)

    return urlunparse((parsed_url[0], parsed_url[1],
                       u'favicon.ico', parsed_url[3],
                       parsed_url[4], parsed_url[5]))


class LocalRemoteUrlNode(template.Node):

    def __init__(self, obj, model_name):
        self.obj = template.Variable(obj)
        self.model_name = model_name

    def render(self, context):
        try:
            object = self.obj.resolve(context)
        except template.VariableDoesNotExist:
            return ''
        try:
            object._meta.get_field_by_name(self.model_name)
        except(FieldDoesNotExist):
            raise template.TemplateSyntaxError('Unknown attribute for %s' % object)

        related = object.__getattribute__(self.model_name)
        if related:
            return related.get_absolute_url()
        else:
            try:
                return object.__getattribute__(self.model_name + '_uri')
            except AttributeError:
                raise FieldError('Field %s_uri is not present on %s' % (self.model_name, object))



@register.tag
def local_or_remote_url(parser, token):
    try:
        tag_name, obj, model_name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, "This tag requires two arguments" 
    #TODO verifier ce qu'on a
    return LocalRemoteUrlNode(obj, model_name)







from django import template
import feedparser

register = template.Library()

class RssParserNode(template.Node):
    def __init__(self, var_name, url=None, url_var_name=None):
        self.url = url
        self.url_var_name = url_var_name
        self.var_name = var_name

    def render(self, context):
        if self.url:
            context[self.var_name] = feedparser.parse(self.url)
        else:
            try:
                context[self.var_name] = feedparser.parse(context[self.url_var_name])
            except KeyError:
                raise template.TemplateSyntaxError, "the variable \"%s\" can't be found in the context" % self.url_var_name
        return ''

import re

@register.tag(name="get_rss")
def get_rss(parser, token):
    # This version uses a regular expression to parse tag contents.
    try:
        # Splitting by None == splitting by spaces.
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires arguments" % token.contents.split()[0]
    
    m = re.search(r'(.*?) as (\w+)', arg)
    if not m:
        raise template.TemplateSyntaxError, "%r tag had invalid arguments" % tag_name
    url, var_name = m.groups()
    
    if url[0] == url[-1] and url[0] in ('"', "'"):
        return RssParserNode(var_name, url=url[1:-1])
    else:
        return RssParserNode(var_name, url_var_name=url)


from time import time

class TimerMiddleware:
    def process_request(self, request):
        request._tm_start_time = time()

    def process_response(self, request, response):
        if not hasattr(request, "_tm_start_time"):
            return

        total = time() - request._tm_start_time

        response['X-Django-Request-Time'] = '%fs' % total
        return response


from django.conf import settings

#: By default we'll set CORS Allow Origin * for all application/json responses
DEFAULT_CORS_PATHS = (
    ('/', ('application/json', ), (('Access-Control-Allow-Origin', '*'), )),
    ('/', ('text/html', ), (('Access-Control-Allow-Origin', '*'), )),
    ('/', ('application/xhtml', ), (('Access-Control-Allow-Origin', '*'), )),
)

class CORSMiddleware(object):
    """
Middleware that serves up representations with a CORS header to
allow third parties to use your web api from JavaScript without
requiring them to proxy it.

See: http://www.w3.org/TR/cors/

Installation
------------

1. Add to ``settings.MIDDLEWARE_CLASSES``::

'sugar.middleware.cors.CORSMiddleware',

2. Optionally, configure ``settings.CORS_PATHS`` if the default settings
aren't appropriate for your application. ``CORS_PATHS`` should be a
list of (path, content_types, headers) values where content_types and
headers are lists of mime types and (key, value) pairs, respectively.

Processing occurs first to last so you should order ``CORS_PATHS``
items from most to least specific.

See ``DEFAULT_CORS_PATHS`` for an example.

Notes
-----

* Although not officially a feature, the headers are not restricted to
the CORS spec and could conceivably include other values such as
desired (allowing, for example, custom ``Cache-Control`` settings).
"""

    def __init__(self):
        self.paths = getattr(settings, "CORS_PATHS", DEFAULT_CORS_PATHS)

    def process_response(self, request, response):
        content_type = response.get('content-type', '').split(";")[0].lower()

        for path, types, headers in self.paths:
            if request.path.startswith(path) and content_type in types:
                for k, v in headers:
                    response[k] = v
                break
        return response


# http://djangosnippets.org/snippets/2753/

from django import forms
from django.db import models
from django.utils.text import capfirst
from django.core import exceptions

from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
from django.utils.encoding import force_unicode
from itertools import chain


class DomainCheckboxSelectMultiple(forms.SelectMultiple):
    def render(self, name, value, attrs=None, choices=()):
        if value is None: 
            value = []
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, name=name)
        output = [u'<ul class="domains">']
        # Normalize to strings
        str_values = set([force_unicode(v) for v in value])
        for i, (option_value, option_label) in enumerate(chain(self.choices, choices)):
            # If an ID attribute was given, add a numeric index as a suffix,
            # so that the checkboxes don't all have the same ID attribute.
            if has_id:
                final_attrs = dict(final_attrs, id='%s_%s' % (attrs['id'], i))
                label_for = u' for="%s"' % final_attrs['id']
            else:
                label_for = ''
            cb = forms.CheckboxInput(final_attrs, check_test=lambda value: value in str_values)
            option_value = force_unicode(option_value)
            rendered_cb = cb.render(name, option_value)
            option_label = conditional_escape(force_unicode(option_label))
            output.append(u'<li><label%s>%s %s</label></li>' % (label_for, rendered_cb, option_label))
        output.append(u'</ul>')
        return mark_safe(u'\n'.join(output))


class MethodsCheckboxSelectMultiple(forms.SelectMultiple):
    def render(self, name, value, attrs=None, choices=()):
        if value is None: 
            value = []
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, name=name)
        output = [u'<ul class="methods">']
        # Normalize to strings
        str_values = set([force_unicode(v) for v in value])

        #methods = {i.id:" ".join(['met'+n for n in i.etypes]) for i in self.choices.queryset}  # for Python > 2.7
        methods = dict([(i.id, " ".join(['met' + n for n in i.etypes])) for i in self.choices.queryset])

        for i, (option_value, option_label) in enumerate(chain(self.choices, choices)):
            # If an ID attribute was given, add a numeric index as a suffix,
            # so that the checkboxes don't all have the same ID attribute.
            if has_id:
                final_attrs = dict(final_attrs, id='%s_%s' % (attrs['id'], i))
                label_for = u' for="%s"' % final_attrs['id']
            else:
                label_for = ''

            label_for = label_for + u' class="%s"' % unicode(methods[option_value])

            cb = forms.CheckboxInput(final_attrs, check_test=lambda value: value in str_values)
            option_value = force_unicode(option_value)
            rendered_cb = cb.render(name, option_value)
            option_label = conditional_escape(force_unicode(option_label))
            output.append(u'<li><label%s>%s %s</label></li>' % (label_for, rendered_cb, option_label))
        output.append(u'</ul>')
        return mark_safe(u'\n'.join(output))


class MultiSelectFormField(forms.MultipleChoiceField):
    widget = MethodsCheckboxSelectMultiple
 
    def __init__(self, *args, **kwargs):
        self.max_choices = kwargs.pop('max_choices', 0)
        super(MultiSelectFormField, self).__init__(*args, **kwargs)
 
    def clean(self, value):
        if not value and self.required:
            raise forms.ValidationError(self.error_messages['required'])
        # if value and self.max_choices and len(value) > self.max_choices:
        #     raise forms.ValidationError('You must select a maximum of %s choice%s.'
        #             % (apnumber(self.max_choices), pluralize(self.max_choices)))
        return value

 
class MultiSelectField(models.Field):
    __metaclass__ = models.SubfieldBase
 
    def get_internal_type(self):
        return "CharField"
 
    def get_choices_default(self):
        return self.get_choices(include_blank=False)
 
    def _get_FIELD_display(self, field):
        value = getattr(self, field.attname)
        choicedict = dict(field.choices)
 
    def formfield(self, **kwargs):
        # don't call super, as that overrides default widget if it has choices
        defaults = {'required': not self.blank, 'label': capfirst(self.verbose_name),
                    'help_text': self.help_text, 'choices': self.choices}
        if self.has_default():
            defaults['initial'] = self.get_default()
        defaults.update(kwargs)
        return MultiSelectFormField(**defaults)

    def get_prep_value(self, value):
        return value

    def get_db_prep_value(self, value, connection=None, prepared=False):
        if isinstance(value, basestring):
            return value
        elif isinstance(value, list):
            return ",".join(value)
 
    def to_python(self, value):
        if value is not None:
            return value if isinstance(value, list) else value.split(',')
        return ''

    def contribute_to_class(self, cls, name):
        super(MultiSelectField, self).contribute_to_class(cls, name)
        if self.choices:
            func = lambda self, fieldname = name, choicedict = dict(self.choices): ",".join([choicedict.get(value, value) for value in getattr(self, fieldname)])
            setattr(cls, 'get_%s_display' % self.name, func)
 
    def validate(self, value, model_instance):
        arr_choices = self.get_choices_selected(self.get_choices_default())
        for opt_select in value:
            if (int(opt_select) not in arr_choices):  # the int() here is for comparing with integer choices
                raise exceptions.ValidationError(self.error_messages['invalid_choice'] % value)  
        return
 
    def get_choices_selected(self, arr_choices=''):
        if not arr_choices:
            return False
        list = []
        for choice_selected in arr_choices:
            list.append(choice_selected[0])
        return list
 
    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        #import pdb; pdb.set_trace()
        return self.get_db_prep_value(value)


from south.modelsinspector import add_introspection_rules  
add_introspection_rules([], ["^coop\.utils\.fields\.MultiSelectField"])




# based on: http://www.djangosnippets.org/snippets/1926/
from django.template import Library, Node, TemplateSyntaxError

register = Library()

class RangeNode(Node):
    def __init__(self, parser, range_args, context_name):
        self.template_parser = parser
        self.range_args = range_args
        self.context_name = context_name

    def render(self, context):

        resolved_ranges = []
        for arg in self.range_args:
            compiled_arg = self.template_parser.compile_filter(arg)
            resolved_ranges.append(compiled_arg.resolve(context, ignore_failures=True))
        context[self.context_name] = range(*resolved_ranges)
        return ""
        
@register.tag
def mkrange(parser, token):
    """
    Accepts the same arguments as the 'range' builtin and creates
    a list containing the result of 'range'.
    
    Syntax:
        {% mkrange [start,] stop[, step] as context_name %}

    For example:
        {% mkrange 5 10 2 as some_range %}
        {% for i in some_range %}
          {{ i }}: Something I want to repeat\n
        {% endfor %}
    
    Produces:
        5: Something I want to repeat 
        7: Something I want to repeat 
        9: Something I want to repeat 
    """

    tokens = token.split_contents()
    fnctl = tokens.pop(0)

    def error():
        raise TemplateSyntaxError, "%s accepts the syntax: {%% %s [start,] " +\
                "stop[, step] as context_name %%}, where 'start', 'stop' " +\
                "and 'step' must all be integers." %(fnctl, fnctl)

    range_args = []
    while True:
        if len(tokens) < 2:
            error()

        token = tokens.pop(0)

        if token == "as":
            break

        range_args.append(token)
    
    if len(tokens) != 1:
        error()

    context_name = tokens.pop()

    return RangeNode(parser, range_args, context_name)


# -*- coding:utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
from django_push.publisher.feeds import Feed
from django.db.models.loading import get_model
from django.contrib.contenttypes.models import ContentType
import datetime


class UpdateFeed(Feed):
    title = _(u"Updates for %s." % Site.objects.get_current().name)
    link = "/feed/" 
    description = _(u"All records updates listed on the %s website." % Site.objects.get_current().name)
    hub = "http://%s/hub/" % Site.objects.get_current()


    def items(self):
        # Call content type pour trouver les models ....
        # ajouter les champs created 
        # ping dans save de model uri
        return get_model(self._mType.app_label, self._model).objects.order_by('-modified')[:5]

    # to deal with overwriting ...
    def item_title(self, item):
        try:
            return item.label()
        except:
            return item.label

    def item_link(self, item):
        return item.uri

    def item_guid(self, item):
        return "%s_%s" % (item.uuid, item.modified)

    def item_pubdate(self, item):
        return datetime.datetime.now()

    def item_description(self, item):
        # return item.uri + 'sparql endpoint' + uriSparql
        return item.uri

    # def item_extra_kwargs
    def get_object(self, request, *args, **kwargs):
        self._model = kwargs['model']
        self._mType = ContentType.objects.get(model=self._model)
        self.title = _(u"Updates for %s on %s." % (self._model, Site.objects.get_current().name))
        self.link = "/feed/%s/" % self._model
        return None







from django.shortcuts import render, render_to_response , get_object_or_404
from django_webid.provider.models import WebIDUser
from django.contrib.auth.models import User
from django.http import Http404
from django.conf import settings

from coop_local.models import Person
from .n3proxy import augment_user_profile

#from django.contrib.auth.decorators import login_required


def people(request, username):
    webiduser = get_object_or_404(WebIDUser, username=username)
    try:
        person = webiduser.get_profile()
    except Person.DoesNotExist:
        raise Http404

    D2RQ_ROOT = settings.D2RQ_ROOT
    profileuri = "%spage/coop_local_person/%s/" % (D2RQ_ROOT, person.id)
    n3uri = "%sdata/coop_local_person/%s/" % (D2RQ_ROOT, person.id)
    rdfgraph = augment_user_profile(webiduser, n3uri).serialize()

    #XXX DEBUG
    return render_to_response('webid/people.html', {
        'request': request,
        'user': request.user,
        'username': username,
        'webiduser': webiduser,
        'person_id': person.id,
        'n3uri': n3uri,
        'profileuri': profileuri,
        'rdfgraph': rdfgraph,
        'STATIC_URL': settings.STATIC_URL,
        'MEDIA_URL': settings.MEDIA_URL})


#@login_required
#XXX is this decodator working as expected?
#I think we need a webidlogin_required...
def test_login(request):
    return render_to_response('webid/auth/testlogin.html', {
        'request': request,
        'user': request.user,
        'webidinfo': getattr(request, 'webidinfo', None),
        'STATIC_URL': settings.STATIC_URL,
        'MEDIA_URL': settings.MEDIA_URL,
        })


def webidlogin_report(request):
    if settings.DEBUG:
        content_type = "text/xml"
    else:
        content_type = "application/rdf+xml"
    return render(request,
        'webid/auth/webidloginReport.html',
        {'user': request.user,
         'request': request,
         'webidinfo': request.webidinfo},
         content_type=content_type,
        )



from django.conf.urls.defaults import *
import views

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    #XXX nameit, we can use it for the WEBIDAUTH_LOGIN_URL
    #(and that, in turn, for the decorator)
    url(r'^test/login$', views.test_login),
    url('^test/WebIDTest', views.webidlogin_report),
)



# -*- coding:utf-8 -*-
from django.contrib import admin
from django import forms
from coop.org.admin import create_action, ContactInline, OrgInline
from django.db.models.loading import get_model

from chosen import widgets as chosenwidgets
from coop.utils.autocomplete_admin import FkAutocompleteAdmin


class PersonAdminForm(forms.ModelForm):
    # category = chosenforms.ChosenModelMultipleChoiceField(
    #         required=False,
    #         overlay=_(u"Choose one or more categories"),
    #         queryset=get_model('coop_local', 'PersonCategory').objects.all())

    class Meta:
        widgets = {'category': chosenwidgets.ChosenSelectMultiple()}

    def __init__(self, *args, **kwargs):
        super(PersonAdminForm, self).__init__(*args, **kwargs)
        self.fields['category'].help_text = None
        self.fields['location'].help_text = None


class PersonAdmin(FkAutocompleteAdmin):
    change_form_template = 'admintools_bootstrap/tabbed_change_form.html'
    related_search_fields = {'location': ('label', 'adr1', 'adr2', 'zipcode', 'city'), }

    form = PersonAdminForm
    search_fields = ['last_name', 'first_name', 'email']
    list_display = ('last_name', 'first_name', 'email', 'structure', 'has_user_account', 'has_role')
    list_filter = ('category',)
    list_display_links = ('last_name', 'first_name')
    search_fields = ('last_name', 'first_name', 'email', 'structure')
    ordering = ('last_name',)
    inlines = [ ContactInline,
                OrgInline,
                ]

    def get_actions(self, request):
        myactions = dict(create_action(s) for s in get_model('coop_local', 'PersonCategory').objects.all())
        return dict(myactions, **super(PersonAdmin, self).get_actions(request))  # merging two dicts

    fieldsets = (
        ('Identification', {
            'fields': (('first_name', 'last_name'),
                        ('location', 'location_display'),  # Using coop-geo
                        'email',
                        'category'
                        ),
            }),
        ('Notes', {
            'fields': ('structure', 'notes',)
        })
    )




import rdflib
import requests
from StringIO import StringIO


def init_n3_graph(n3uri):
    """
    initializes a rdflib conjunctive graph with content from
    a given n3 uri.
    """
    #XXX TODO: rewrite the D2RQ URIS
    graph = rdflib.ConjunctiveGraph()
    req = requests.get(n3uri)
    #XXX TODO:
    #Catch BadSyntax errors 
    #(when we get a 500 from d2rq, for instance.
    #So we probably should check for response code
    #and stuff...
    graph.parse(StringIO(req.content), format="n3")
    return graph


def add_key_triples_to_graph(key, graph, person_uri):
    """
    adds the triples containing info about the
    rsa key
    """
    CERT = rdflib.Namespace("http://www.w3.org/ns/auth/cert#")

    person = rdflib.term.URIRef(person_uri)

    rpk = rdflib.BNode()
    graph.add((rpk, rdflib.RDF.type, CERT["RSAPublicKey"]))

    #XXX FIXME check data types and stuff
    graph.add((rpk, CERT["modulus"], rdflib.Literal(key.mod,
        datatype="http://www.w3.org/2001/XMLSchema#hexBinary")))
    graph.add((rpk, CERT["exponent"], rdflib.Literal(key.exp,
        datatype="http://www.w3.org/2001/XMLSchema#integer")))
    graph.add((person, CERT["key"], rpk))

    return graph


def augment_user_profile(webiduser, n3uri):
    graph = init_n3_graph(n3uri)
    person_id = webiduser.get_profile().id
    person_resource_uri = "http://localhost:8080/resource/coop_local_person/%s/" % (person_id)

    #XXX FIXME fix building this, HARDCODED!
    #XXX actually, should rewrite or something...

    for key in webiduser.keys:
        graph = add_key_triples_to_graph(key,
                graph,
                person_resource_uri)
    return graph



# -*- coding:utf-8 -*-

import os
from coop_local.settings import PROJECT_PATH, PROJECT_NAME

D2RQ_PORT = 8080
D2RQ_ROOT = 'http://localhost:8080/' + PROJECT_NAME + '/'

PES_HOST = 'http://localhost:8040/'

TIME_ZONE = 'Europe/Paris'
LANGUAGE_CODE = 'fr-FR'
SITE_ID = 1
USE_I18N = True
USE_L10N = True

DEFAULT_CONTENT_TYPE = 'text/html'
DEFAULT_CHARSET = 'utf-8'

LOGIN_URL = '/accounts/login/'
LOGIN_REDIRECT_URL = '/'

# Not used: to set this variable here, weneed to import Site
PUSH_HUB = ''
SUBHUB_MAINTENANCE_AUTO = False

USE_TZ = False

# Upload directory
MEDIA_ROOT = os.path.abspath(PROJECT_PATH + '/media/')
MEDIA_URL = '/media/'

# Static files
STATIC_ROOT = os.path.abspath(PROJECT_PATH + '/static_collected/')
STATIC_URL = '/static/'

# compat fix ?
ADMIN_MEDIA_PREFIX = '/static/admin/'


import admin_tools
ADMIN_TOOLS_PATH = os.path.dirname(os.path.abspath(admin_tools.__file__))


STATICFILES_DIRS = [
    os.path.abspath(ADMIN_TOOLS_PATH + '/media/'),
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
]

TEMPLATE_DIRS = [
    os.path.abspath(PROJECT_PATH + '/coop_local/templates/')
]

TEMPLATE_LOADERS = [
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
]

MIDDLEWARE_CLASSES = [
    #'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.gzip.GZipMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    #'django.middleware.cache.FetchFromCacheMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django_webid.auth.middleware.WEBIDAuthMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'coop.utils.middleware.CORSMiddleware',
    'pagination.middleware.PaginationMiddleware',
    'raven.contrib.django.middleware.Sentry404CatchMiddleware',
]

TEMPLATE_CONTEXT_PROCESSORS = [
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.request',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.tz',
    'django.contrib.messages.context_processors.messages',
    'coop.context_processors.current_site',
    'coop.context_processors.d2rq_settings',
]

ROOT_URLCONF = 'coop_local.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'coop_local.wsgi.application'

# Cache middelware must be enabled above
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    },
    'memcached': {
        'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
        #'LOCATION': '127.0.0.1:11211',
        'LOCATION': 'unix:/tmp/memcached.sock',
    }
}

CACHE_MIDDLEWARE_ALIAS = 'memcached'
CACHE_MIDDLEWARE_SECONDS = 60 * 60
CACHE_MIDDLEWARE_KEY_PREFIX = PROJECT_NAME


INSTALLED_APPS = [

    # admin tools
    'admintools_bootstrap',
    'admin_tools',
    'admin_tools.theming',
    'admin_tools.menu',
    'admin_tools.dashboard',

    # django contribs
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.gis',
    'django.contrib.comments',

    # other 3rd parties
    'south',
    'django_extensions',
    'extended_choices',
    'floppyforms',
    'django_rq',
    'subhub',
    #'haystack',
    #'oembed',
    'chosen',
    'sorl.thumbnail',
    'tinymce',

    # coop_cms
    'djaloha',
    'coop_cms',
    'colorbox',
    'coop_bar',
    'pagination',

    # WebID
    'django_webid.provider',
    'django_webid.auth',

    # tags
    'coop_tag',
    'taggit',
    'taggit_templatetags',
    'taggit_autosuggest',

    # PUSH
    # 'django_push.subscriber',
    'uriredirect',

    # Logging errors (needed until we go "stable")
    'raven.contrib.django',

]



from coop.settings import DEFAULT_RDF_NAMESPACES, NS_D2RQ, NS_LEGAL, NS_SKOS, NS_OV
from coop.settings import NS_RDFS
RDF_NAMESPACES = DEFAULT_RDF_NAMESPACES

THUMBNAIL_FORMAT = 'PNG'

TINYMCE_DEFAULT_CONFIG = {
    'theme': "advanced",
    'relative_urls': False,
    'width': '617px', 'height': '220px',
    'theme_advanced_toolbar_location': 'top',
    'theme_advanced_buttons1': 'bold,italic,|,justifyleft,justifycenter,justifyright,|,bullist,numlist,|,link,unlink,|,code',
    'theme_advanced_buttons2': '', 'theme_advanced_buttons3': ''
    }

COOP_CMS_ARTICLE_CLASS = 'coop_local.models.Article'
COOP_CMS_ARTICLE_FORM = 'coop.forms.ArticleForm'
COOP_CMS_ARTICLE_LOGO_SIZE = '600'

# You need to declare the app here to be able to select models from navigable types in coop-cms
COOP_CMS_CONTENT_APPS = ('coop_local', 'coop_tag', 'coop_geo', 'coop_cms')

DJALOHA_LINK_MODELS = (
        'coop_local.Article',
        'coop_cms.ArticleCategory',
        'coop_local.Organization',
        'coop_local.OrganizationCategory'
        )



# COOP_CMS_ARTICLE_TEMPLATES = 'coop_local.get_article_templates' # marche plus ?

COOP_CMS_ARTICLE_TEMPLATES = [
    ('coop_cms/article_standard.html', 'Standard'),
    ('coop_cms/article_rubrique.html', 'Rubrique'),
]

COOPBAR_MODULES = [
    'coop_cms.coop_bar_cfg',
    'coop.coop_bar_cfg'
    ]


HAYSTACK_SITECONF = 'coop_local.search_sites'
HAYSTACK_SEARCH_ENGINE = 'simple'
HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.simple_backend.SimpleEngine',
    },
}

ADMIN_TOOLS_MENU = 'coop.ui.menu.CustomMenu'
ADMIN_TOOLS_INDEX_DASHBOARD = 'coop.ui.dashboard.CustomIndexDashboard'
ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'coop.ui.dashboard.CustomAppIndexDashboard'
ADMIN_TOOLS_THEMING_CSS = 'css/coop_bootstrap_theming.css'


AUTHENTICATION_BACKENDS = [
    'django_webid.auth.backends.WEBIDAuthBackend',
    'coop_cms.perms_backends.ArticlePermissionBackend',
    'django.contrib.auth.backends.ModelBackend',
]

AUTH_PROFILE_MODULE = 'coop_local.Person'

# projection used in database
COOP_GEO_EPSG_PROJECTION = 4326  # WGS84

COOP_GEO_BOUNDING_BOX = []
COOP_GEO_REGION = LANGUAGE_CODE[:2]

TAGGIT_TAG_MODEL = ('coop_tag', 'Ctag')
TAGGIT_TAGGED_ITEM_MODEL = ('coop_tag', 'CtaggedItem')
TAGGIT_TAG_FIELD_RELATED_NAME = 'ctagged_items'


TAGGIT_AUTOCOMPLETE_TAG_MODEL = 'coop_tag.Ctag'
# DEFAULT_TAGGIT_AUTOCOMPLETE_MEDIA_URL =


TAGGIT_AUTOSUGGEST_MODEL = ('coop_tag', 'Ctag')
TAGGIT_AUTOSUGGEST_CSS_FILENAME = 'coop_tag.css'


#
# WebID Options
#

WEBIDAUTH_USE_COOKIE = True

# The following lines set the local user creation callback.
# All the info from the remote profile is accessible from
# the request.webidinfo object (see documentation).


def createusercb(req):
    from coop.webid.utils import build_coop_user
    return build_coop_user(req)

WEBIDAUTH_CREATE_USER_CALLBACK = createusercb

WEBIDPROVIDER_SKIP_PROFILE_INIT = True

# Uncomment the following lines if you want to specify
# a custom callback for the assigment of the WebIDUris.
# If this callback is not specified, provider will look for
# a webidprovider-webid_uri urlpattern and will try to
# reverse it passing the webiduser instance.

#def webidcb(webiduser):
#    "avoids circular import"
#    from coop.webid.utils import custom_webid_uri
#    return custom_webid_uri(webiduser)
#WEBIDPROVIDER_WEBIDURI_CALLBACK = webidcb



# LOGGING = {
#     'version': 1,
#     'disable_existing_loggers': False,
#     'filters': {
#         'require_debug_false': {
#             '()': 'django.utils.log.RequireDebugFalse'
#         }
#     },
#     'handlers': {
#         'mail_admins': {
#             'level': 'ERROR',
#             'filters': ['require_debug_false'],
#             'class': 'django.utils.log.AdminEmailHandler'
#         }
#     },
#     'loggers': {
#         'django.request': {
#             'handlers': ['mail_admins'],
#             'level': 'ERROR',
#             'propagate': True,
#         },
#     }
# }


LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'root': {
        'level': 'WARNING',
        'handlers': ['sentry'],
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
    },
    'handlers': {
        'sentry': {
            'level': 'ERROR',
            'class': 'raven.contrib.django.handlers.SentryHandler',
       },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        #  'file_subhub': {  # define and name a handler
        #     'level': 'DEBUG',
        #     'class': 'logging.handlers.WatchedFileHandler',  # set the logging class to log to a file
        #     'filename': os.path.abspath(PROJECT_PATH + '/logs/subhub.log')  # log file
        # }

    },
    'loggers': {
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
        },
        'subhub.maintenance': {
            'handlers': ['console'],
            'level': 'DEBUG'
        },
        'subhub.distribution.process': {
            'handlers': ['console'],
            'level': 'DEBUG'

        },
        'raven': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
        'sentry.errors': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
    },

}



# For RQ
RQ_QUEUES = {
    'default': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    },
    'high': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    },
    'low': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    }
}



# # -*- coding:utf-8 -*-
from django.shortcuts import render_to_response, redirect
from coop_local.models import Organization, Person
from django.template import RequestContext
from django.core.urlresolvers import reverse

def perso(request):
    context = {}
    context['user'] = request.user
    return render_to_response('person/panel.html',context,RequestContext(request))


import types

from django.conf import settings
from django.http import Http404
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext, TemplateDoesNotExist
from django.utils.decorators import classonlymethod

from django_conneg.views import ContentNegotiatedView
from django_conneg.decorators import renderer as conneg_renderer


from django_webid.provider.models import WebIDUser

import n3proxy
from coop_local.models import Person


class PriorityContentNegotiatedView(ContentNegotiatedView):
    """
    dynamic addition
    of renderers
    """

    @classonlymethod
    def as_view(cls, **initkwargs):
        ranked_mimetypes = getattr(cls, '_mimetypes', None)
        if ranked_mimetypes:
            for mime, prio, name in ranked_mimetypes:
                renderer = cls.getRenderer(name, mime, name, prio)
                cls.addMethod(renderer)
        view = super(PriorityContentNegotiatedView, cls).as_view(**initkwargs)
        return view

    @classonlymethod
    def addMethod(cls, func):
        return setattr(cls, func.__name__, types.MethodType(func, cls))

    @classonlymethod
    def getRenderer(cls, format, mimetypes, name, priority):
        if not isinstance(mimetypes, tuple):
            mimetypes = (mimetypes,)
        mime = mimetypes[0]
        
        def renderer(cls, self, request, context, template_name):
            template_name = self.join_template_name(template_name, name.lower())
            if template_name is None:
                return NotImplemented
            try:
                return render_to_response(template_name,
                                          context,
                                          context_instance=RequestContext(request),
                                          mimetype=mime)
            except TemplateDoesNotExist:
                return NotImplemented

        renderer.__name__ = 'render_%s' % mime.replace('/', '_')
        renderer = conneg_renderer(format=format,
                                       mimetypes=mimetypes,
                                       priority=priority)(renderer)
        return renderer


class WebIDProfileView(PriorityContentNegotiatedView):
    """
    View that negotiates the output format
    Supports: html, rdfa, rdf+xml, xhtml, turtle
    ... why not vcard???
    """
    _default_format = 'html'
    #fallback format???
    #_force_fallback_format = 'html'
    _mimetypes = (('text/html', 10, 'html'),
                  ('application/xhtml+xml', 5, 'rdfa'),
                  ('application/rdf+xml', 1, 'rdfxml'),
                  ('text/turtle', 2, 'turtle'))

    def get(self, request, username=None):
        webiduser = get_object_or_404(WebIDUser,
            username=username)
        try:
            person = webiduser.get_profile()
        except Person.DoesNotExist:
            raise Http404
        D2RQ_ROOT = settings.D2RQ_ROOT
        #profileuri = "%spage/coop_local_person/%s/" % (D2RQ_ROOT, person.id)
        n3uri = "%sdata/coop_local_person/%s/" % (D2RQ_ROOT, person.id)
        rdfgraph = n3proxy.augment_user_profile(webiduser, n3uri).serialize()

        context = {
                "webiduser": webiduser,
                "xmlpayload": rdfgraph,
                "MEDIA_URL": settings.MEDIA_URL,
                "STATIC_URL": settings.STATIC_URL,
        }
        # Call render, passing a template name (w/o file extension)
        return self.render(request, context,
                'webid/profiles/webid')

    # fix head method
    # (fixed in bennomadic fork of django_conneg, which is now
    # in requirements, let's see if it's merged upstream.)

    #def head(self, request, *args, **kwargs):
    #    return self.get(request, *args, **kwargs)



# -*- coding:utf-8 -*-
from django.db import models
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from extended_choices import Choices
from django.core.urlresolvers import reverse
from decimal import Decimal
from django.conf import settings
from coop.models import URIModel
from coop.utils.fields import MultiSelectField
import rdflib

if "coop_geo" in settings.INSTALLED_APPS:
    from coop_geo.models import Area, Location


class BaseProduct(URIModel):
    title = models.CharField(_('title'), blank=True, max_length=250)
    slug = exfields.AutoSlugField(populate_from='title', overwrite=True)
    description = models.TextField(_(u'description'), blank=True)
    organization = models.ForeignKey('coop_local.Organization', blank=True, null=True,
                                        verbose_name='publisher', related_name='products')
    organization_uri = models.CharField(_(u'publisher URI'), blank=True, max_length=200, editable=False)

    def __unicode__(self):
        return self.title + u' (' + self.organization.__unicode__() + u')'

    class Meta:
        abstract = True
        ordering = ['-modified']
        verbose_name = _(u'Product')
        verbose_name_plural = _(u'Products')


EWAY = Choices(
    ('OFFER',   1,  _(u"I'm offering")),
    ('NEED',    2,  _(u"I'm looking for"))
)

ETYPE = Choices(
    ('PROD',    1,  _(u'Product or Material')),
    ('SERVE',   2,  _(u'Service')),
    ('SKILL',   3,  _(u'Skill')),
    ('COOP',    4,  _(u'Partnership')),
    ('QA',      5,  _(u'Question')),
)


class BaseExchangeMethod(models.Model):  # this model will be initialized with a fixture
    label = models.CharField(_(u'label'), max_length=250)
    uri = models.CharField(_(u'URI'), blank=True, max_length=250)
    etypes = MultiSelectField(_(u'applicable to'), max_length=250, null=True, blank=True, choices=ETYPE.CHOICES)

    def applications(self):
        return u", ".join([ETYPE.CHOICES_DICT[int(e)].__unicode__() for e in self.etypes])
    applications.short_description = _(u'applications')

    def __unicode__(self):
        return self.label

    class Meta:
        abstract = True
        verbose_name = _(u'Exchange method')
        verbose_name_plural = _(u'Exchange methods')


class BaseExchange(URIModel):
    title = models.CharField(_('title'), max_length=250)
    description = models.TextField(_(u'description'), blank=True)
    organization = models.ForeignKey('coop_local.Organization', blank=True, null=True,
                            verbose_name=_('publisher'), related_name='exchanges')
    person = models.ForeignKey('coop_local.Person', blank=True, null=True, verbose_name=_(u'person'))

    eway = models.PositiveSmallIntegerField(_(u'exchange way'), choices=EWAY.CHOICES, default=EWAY.OFFER)
    etype = models.PositiveSmallIntegerField(_(u'exchange type'), choices=ETYPE.CHOICES)

    permanent = models.BooleanField(_(u'permanent'), default=True)
    expiration = models.DateField(_(u'expiration'), blank=True, null=True)
    slug = exfields.AutoSlugField(populate_from='title', overwrite=True)
    products = models.ManyToManyField('coop_local.Product', verbose_name=_(u'linked products'))
    person_uri = models.CharField(_(u'person URI'), blank=True, max_length=200, editable=False)
    organization_uri = models.CharField(_(u'publisher URI'), blank=True, max_length=200, editable=False)

    methods = models.ManyToManyField('coop_local.ExchangeMethod', verbose_name=_(u'exchange methods'))


    # coop_geo must be loaded BEFORE coop_local
    if "coop_geo" in settings.INSTALLED_APPS:

        location = models.ForeignKey(   Location,
                                        verbose_name=_(u'location'),
                                        null=True, blank=True, related_name='exchange_location',
                                        help_text=_("choose a location among your registered locations."))
        area = models.ForeignKey(Area,
                                 verbose_name=_(u'interest area'),
                                 null=True, blank=True, related_name='exchange_area',
                                 help_text=_("choose an area among your registered impact areas."))


    def __unicode__(self):
        return unicode(self.title)

    def get_absolute_url(self):
        return reverse('exchange_detail', args=[self.id])

    def label(self):
        return self.title

    #TODO assign the record to the person editing it (form public) and provide an A-C choice in admin

    class Meta:
        abstract = True
        ordering = ('-modified',)
        verbose_name = _(u'Exchange')
        verbose_name_plural = _(u'Exchanges')

    # title field needs a special handling. checkDirectMap does not work
    # because two rdf property use the coop_local_organization.title field
    # Thus we have to decide which one to use
    def updateField_title(self, dbfieldname, graph):
            title = list(graph.objects(rdflib.term.URIRef(self.uri), settings.NS_RDFS.label))
            if len(title) == 1:
                self.title = title[0]
                print "For id %s update the field %s" % (self.id, dbfieldname)
            else:
                print "    The field %s cannot be updated." % dbfieldname



class BaseTransaction(models.Model):
    origin = models.ForeignKey('coop_local.Exchange', related_name='origin', verbose_name=_(u'origin'))
    destination = models.ForeignKey('coop_local.Exchange', related_name='destination', verbose_name=_(u'destination'))
    origin_org = models.ForeignKey('coop_local.Organization', related_name='contrats_vente', verbose_name=_(u'vendor'), blank=True, null=True)
    destination_org = models.ForeignKey('coop_local.Organization', related_name='contrats_achat', verbose_name=_(u'buyer'), blank=True, null=True)
    title = models.CharField(_('title'), blank=True, max_length=250)
    description = models.TextField(_(u'description'), blank=True)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    uuid = exfields.UUIDField()  # nécessaire pour URI ?

    def __unicode__(self):
        return self.title

    class Meta:
        abstract = True
        verbose_name = _(u'Transaction')
        verbose_name_plural = _(u'Transactions')


# MODALITIES = Choices(
#     ('GIFT',    1,  _(u'Gift')),
#     ('TROC',    2,  _(u'Free exchange')),
#     ('CURR',    3,  _(u'Monetary exchange')),
# )
# UNITS = Choices(
#     ('EURO',    1,  _(u'€')),
#     ('SELH',    2,  _(u'Hours')),
#     ('PEZ',     3,  _(u'PEZ')),
# )


# class BasePaymentModality(models.Model):
#     exchange = models.ForeignKey('coop_local.Exchange', verbose_name=_(u'exchange'), related_name='modalities')
#     modality = models.PositiveSmallIntegerField(_(u'exchange type'), blank=True,
#                                                 choices=MODALITIES.CHOICES,
#                                                 default=MODALITIES.CURR)
#     amount = models.DecimalField(_(u'amount'), max_digits=12, decimal_places=2, default=Decimal(0.00), blank=True)
#     unit = models.PositiveSmallIntegerField(_(u'unit'), blank=True, null=True, choices=UNITS.CHOICES)

#     def __unicode__(self):
#         if(self.modality in [1, 2]):
#             return unicode(MODALITIES.CHOICES_DICT[self.modality])
#         elif(self.modality == 3 and self.amount > 0 and not self.unit == None):
#             return unicode("%s %s" % (self.amount, unicode(UNITS.CHOICES_DICT[self.unit])))
#         elif(self.modality == 3):
#             return unicode(_(u'Price unknown'))

#     def save(self, *args, **kwargs):
#         if not self.modality == 3:
#             self.amount = Decimal(0.00)
#             self.unit = None
#         super(BasePaymentModality, self).save(*args, **kwargs)

#     class Meta:
#         abstract = True
#         verbose_name = _(u'Payment modality')
#         verbose_name_plural = _(u'Payment modalities')







# -*- coding:utf-8 -*-
from django.conf.urls.defaults import patterns, include, url

urlpatterns = patterns('',
    (r'^(?P<model>[a-z]+)/(?P<uuid>\w+).(?P<format>\w+)$', 'coop.views.get_rdf'),  # [xml,n3,ttl,json]
)




# # -*- coding:utf-8 -*-
        
from django.shortcuts import render_to_response
from coop_local.models import Organization, OrganizationCategory, Engagement, Role
from django.template import RequestContext
from django.shortcuts import get_object_or_404


def org_category_detail(request, slug):
    cat = get_object_or_404(OrganizationCategory, slug=slug) 
    org_list = Organization.objects.filter(category=cat)   
    context = {'category': cat}
    context['org_list'] = org_list
    return render_to_response('org/org_category.html', context, RequestContext(request))


def org_detail_view(request, slug):
    context = {'editable': True}
    org = get_object_or_404(Organization, slug=slug)    
    context['organization'] = org
    context['object'] = org
    #context['rss'] = org.contact.get(category=6)
    context['adresses'] = org.located.all()
    context['engagements'] = Engagement.objects.filter(organization=org)
    return render_to_response('org/org_detail.html', context, RequestContext(request))


def org_edit(request, slug):
    context = {}
    org = get_object_or_404(Organization, slug=slug)    
    context['object'] = org
    context['adresses'] = org.located.all()
    context['engagements'] = Engagement.objects.filter(organization=context['object'])
    return render_to_response('org/org_detail.html', context, RequestContext(request))
   

def org_list(request):
    context = {}
    context['org_list'] = Organization.objects.filter(active=True)
    return render_to_response('org/org_list.html', context, RequestContext(request))
        
    
def role_detail(request, slug):
    context = {}
    role = Role.objects.get(slug=slug)
    context['object'] = role
    context['engagements'] = Engagement.objects.filter(role=role).select_related('person').order_by('person__last_name')
    return render_to_response('org/role_detail.html', context, RequestContext(request))


def global_map(request):
    context = {}
    context['initiatives'] = Organization.objects.filter(active=True)
    return render_to_response('org/org_global_map.html',
                              context, RequestContext(request))

def leaflet(request):
    context = {}
    context['org_list'] = Organization.objects.filter(active=True)
    return render_to_response('org/org_carto.html', context, RequestContext(request))



from django.db import models
import coop
import shortuuid

ll = models.get_models()

for l in ll:
    if l.__mro__.__contains__(coop.models.URIModel):
        print 'Je fait un save sur les instances du models %s' % l
        for o in l.objects.all():
            if o.uuid == None or "-" in o.uuid:
                o.uuid = shortuuid.uuid()
            o.save()



import logging
logger = logging.getLogger('commerce')

class LimitedAdminInlineMixin(object):
    """
    InlineAdmin mixin limiting the selection of related items according to
    criteria which can depend on the current parent object being edited.

    A typical use case would be selecting a subset of related items from
    other inlines, ie. images, to have some relation to other inlines.

    Use as follows::

        class MyInline(LimitedAdminInlineMixin, admin.TabularInline):
            def get_filters(self, obj):
                return (('<field_name>', dict(<filters>)),)

    """

    @staticmethod
    def limit_inline_choices(formset, field, empty=False, **filters):
        """
        This function fetches the queryset with available choices for a given
        `field` and filters it based on the criteria specified in filters,
        unless `empty=True`. In this case, no choices will be made available.
        """
        assert formset.form.base_fields.has_key(field)
        qs = formset.form.base_fields[field].queryset
        if empty:
            logger.debug('Limiting the queryset to none')
            formset.form.base_fields[field].queryset = qs.none()
        else:
            qs = qs.filter(**filters)
            logger.debug('Limiting queryset for formset to: %s', qs)            

            formset.form.base_fields[field].queryset = qs

    def get_formset(self, request, obj=None, **kwargs):
        """
        Make sure we can only select variations that relate to the current
        item.
        """        
        formset = \
            super(LimitedAdminInlineMixin, self).get_formset(request,
                                                             obj,
                                                             **kwargs)
        for (field, filters) in self.get_filters(obj):
            self.limit_inline_choices(formset, field, **filters)
            #if obj:
            #    self.limit_inline_choices(formset, field, **filters)
            #else:
            #    self.limit_inline_choices(formset, field, empty=True)

        return formset

    def get_filters(self, obj):
        """
        Return filters for the specified fields. Filters should be in the
        following format::

            (('field_name', {'categories': obj}), ...)

        For this to work, we should either override `get_filters` in a
        subclass or define a `filters` property with the same syntax as this
        one.
        """
        return getattr(self, 'filters', ())



from django.conf import settings
from django.contrib.sites.models import Site

def current_site(request):
    '''
    A context processor to add the "current site" to the current Context
    '''
    try:
        current_site = Site.objects.get_current()
        return {
            'current_site': current_site,
            'site_author': settings.SITE_AUTHOR,
            'site_title': settings.SITE_TITLE,
        }
    except Site.DoesNotExist:
        # always return a dict, no matter what!
        return {'current_site':''} # an empty string
        
def d2rq_settings(request):
    return {'D2RQ_ROOT' : settings.D2RQ_ROOT}        


# -*- coding:utf-8 -*-
from django.db import models
from django_extensions.db import fields as exfields
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from extended_choices import Choices
import shortuuid
from rdflib import Graph, plugin, store, Literal, URIRef
from django.db import IntegrityError
from django.template import Template, Context
from urlparse import urlsplit
import feedparser
import os
import tempfile
import coop
import time




URI_MODE = Choices(
    ('LOCAL',  1, _(u'Local')),
    ('COMMON',   2, _(u'Common')),
    ('IMPORTED', 3, _(u'Imported')),
)

# Store
plugin.register('SPARQLStore', store.Store,
        'rdflib_sparqlstore.SPARQLStore', 'SPARQLStore')

# Serializer
plugin.register('n3', plugin.Serializer,
         'rdflib.plugins.serializers.n3', 'N3Serializer')
plugin.register('pretty-xml', plugin.Serializer,
         'rdflib.plugins.serializers.rdfxml', 'PrettyXMLSerializer')
plugin.register('json-ld', plugin.Serializer,
        'rdflib_jsonld.jsonld_serializer', 'JsonLDSerializer')


class TimestampedModel(models.Model):
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)

    class Meta:
        abstract = True


class StaticURIModel(models.Model):
    """
    To use this model as a basis for your own abstract model, you need to have
    a 'uri_id' property set and a string for the model type URL representation :

    @property
    def uri_id(self):
        return self.username # example with username

    uri_registry = 'org'

    the uri_fragment can then be overriden in a real model derived from your abstract model

    """
    class Meta:
        abstract = True

    uri_mode = models.PositiveSmallIntegerField(_(u'Mode'), choices=URI_MODE.CHOICES, default=URI_MODE.LOCAL, editable=False)
    uri = models.CharField(_(u'main URI'), blank=True, null=True,
                            max_length=250, editable=False)  # FIXME : null=True incompatible with unique=True

    # Le code suivante ne marche pas avec south et pourtant il correxpond exactement à ce que je voudrais
    # faire. Si on supprime unique=True, alors la migration se passe bien, mais toutes les
    # du model ont la meme valeur.... Et bien sur, si on met uniaue=True, south rale car
    # le champs n'est pas unique.
    # South n'a pas l'air capable de traiter ce cas.
    # D'apres la doc, il est possible de le faire en 3 fois, sans utiliser le default, avec
    # un datamigration (voir la doc Part 3: Advanced Commands and Data Migrations)
    #uuid = models.CharField(_(u'uuid'), max_length=50, default=shortuuid.uuid, unique=True)

    # La version simple c'est de passer par le save() et de supprimer le default.... c'est pas
    # tres beau, car un plus couteux en runtime... mais bon
    uuid = models.CharField(_(u'uuid'), max_length=50, unique=True, null=True, default=shortuuid.uuid, editable=False)

    # the default value, this attribut should be overloaded
    domain_name = settings.DEFAULT_URI_DOMAIN
    #str(Site.objects.get_current().domain)

    def label(self):
        return "NYI label method"

    # This metho could be overwritten by subClasses
    @property
    def uri_id(self):
        return self.uuid

    def uri_registry(self):
        return self.__class__.__name__.lower()

    def init_uri(self):
        return 'http://' + self.domain_name + '/id/' + self.uri_registry() + '/' + str(self.uri_id) + '/'




    # self.uri est null ou vide le creer (cad appel init_uri)
    # si le domain a changé alors changer l'uri (cad dire appel init_uri)
    # si le uri_registery a changer alors changer l'uri
    # attention dans les 2 cas le uuid ne bouge pas.... il faut alors aller le chercher
    #
    # repenser aux commons et aux importés pour voir si tout marche



    def save(self, *args, **kwargs):
         # create / update URI
        if self.uri_mode != URI_MODE.IMPORTED:
            if not self.uri or self.uri == '':
                self.uri = self.init_uri()
            else:
                scheme, host, path, query, fragment = urlsplit(self.uri)
                sp = path.split('/')
                try:
                    assert(sp[1] == 'id')  # to assert a minimal coherence...
                except AssertionError:
                    raise IntegrityError(_(u'Local URI path must starts with "/id/"'))
                if sp[3] != self.uri_id:
                    self.uri = self.init_uri()  # uri_id est pas pareil
                else:
                    if host != self.domain_name:
                        self.uri = self.init_uri()
                    else:
                        if  sp[2] != self.uri_registry():
                            self.uri = self.init_uri()
        super(StaticURIModel, self).save(*args, **kwargs)



    def toRdf(self, format):
        """
           format correspond to the standard rdflib format keyword,
               'n3' for n3
               'xml' for xml
               'json-ld' for json-ld
        """
        # Sparq endPoint
        uriSparql = 'http://localhost:8080/' + settings.PROJECT_NAME + '/sparql'
        graph = Graph(store="SPARQLStore")
        graph.open(uriSparql, False)
        graph.store.baseURI = str(uriSparql)

        # The short form of the construct where see 16.2.4 CONSTRUCT WHERE http://www.w3.org/TR/sparql11-query/#constructWhere
        cquery = "construct where { <%s> ?p ?o .} "
        res = graph.query(cquery % self.uri)
        filename = "/tmp/%stmp.rdf" % self.uuid
        tmpfile = open(filename, "w")
        res.serialize().write(tmpfile, encoding='utf-8')
        tmpfile.close()
        g = Graph()
        for p, ns in settings.RDF_NAMESPACES.iteritems():
            g.bind(p, ns)
        g.parse(filename)

        if format == 'ttl': format = 'n3'
        if format == 'json': format = 'json-ld'

        return g.serialize(format=format)

    def toN3(self):
        return self.toRdf("n3")

    def toXml(self):
        return self.toRdf("xml")

    def toJson(self):
        return self.toRdf("json-ld")


    @classmethod
    def getD2rqGraph(cls):
        context = {}
        context['mapping_template'] = 'd2r/' + cls.__name__.lower() + '.ttl'
        context['namespaces'] = settings.RDF_NAMESPACES

        tmpfile = os.path.dirname(os.path.abspath(coop.__file__)) + '/templates/d2r/model.ttl'
        f = open(tmpfile, 'r')
        t = Template(f.read())
        f.close()
        res = t.render(Context(context))
        (fd, fname) = tempfile.mkstemp()
        f = open(fname, 'w')
        f.write(res)
        f.close()
        graph = Graph()
        graph.parse('file:' + fname, format='n3')
        return graph




    @classmethod
    def updateFromFeeds(cls):
        print "Update feed for class name %s" % cls.__name__
        feed_url = settings.PES_HOST + 'feed/' + cls.__name__.lower() + '/'
        parsedFeed = feedparser.parse(feed_url)
        print "Parse feed %s" % feed_url
        for entry in parsedFeed.entries:
            print "Entry summary is %s " % entry.summary
            fd, fname = tempfile.mkstemp()
            os.write(fd, entry.summary)
            os.close(fd)
            g = Graph()
            g.parse(fname, format="json-ld")
            done = set()
            for subj in g.subjects(None, None):
                if not (subj in done):
                    instance = cls.objects.get(uri=str(subj))
                    if not instance:
                        print "Nothing to do with uri %s. Not found in database" % subj
                    else:
                        instance.updateFromRdf(g)
                    done.add(subj)

    # The "reverse mapping is done here"
    def updateFromRdf(self, graph):
        db_table = self.__class__._meta.db_table
        d2rqGraph = self.__class__.getD2rqGraph()
        for field in self.__class__._meta.fields:
            dbfieldname = db_table + '.' + field.name
            pred = checkDirectMap(dbfieldname, d2rqGraph)
            if pred:
                update = list(graph.objects(URIRef(self.uri), pred))
                if len(update) > 1:
                    print "    The field %s cannot be updated. Too many values" % dbfieldname
                elif len(update) == 0:
                    print "Nothing to update for field %s" % dbfieldname
                else:
                    print "For id %s update the field %s" % (self.id, dbfieldname)
                    update = update[0]
                    # Well some field have to be prepared
                    if isinstance(field, models.fields.DateField):
                        update = update.split('.')[0]
                        tt = time.strptime(update, "%Y-%m-%dT%H:%M:%S")
                        update = "%d-%d-%d" % (tt.tm_year, tt.tm_mon, tt.tm_mday)
                    setattr(self, field.name, update)
            else:
                 # See org/models file to have example of "special" fiels handling
                if hasattr(self, 'updateField_' + field.name):
                    getattr(self, 'updateField_' + field.name)(dbfieldname, graph)
                else:
                    print "    The field %s cannot be updated." % dbfieldname
        self.save()




class URIModel(StaticURIModel, TimestampedModel):
    class Meta:
        abstract = True






def checkDirectMap(dbfieldName, d2rqGraph):
    lit = Literal(dbfieldName)
    subj = list(d2rqGraph.subjects(settings.NS_D2RQ.column, lit))
    if len(subj) != 1:
        subj = list(d2rqGraph.subjects(settings.NS_D2RQ.uriColumn, lit))
        if len(subj) != 1:
            return None
    subj = subj[0]
    prop = list(d2rqGraph.objects(subj, settings.NS_D2RQ.property))
    if len(prop) != 1:
        return None
    else:
        return prop[0]


# It seems to be the best place to do the connection
# see http://stackoverflow.com/questions/7115097/the-right-place-to-keep-my-signals-py-files-in-django
from django.core.signals import  request_finished
from coop.signals import post_save_callback
request_finished.connect(post_save_callback, sender=StaticURIModel)



# -*- coding:utf-8 -*-
# rssplug.py
# Alberto García Hierro
# Drop this file inside your templatetags folder
# See http://fi.am/entry/plugging-a-rss-feed-into-a-django-template/

from datetime import datetime
import time

from django.core.cache import cache
from django import template

import feedparser

register = template.Library()

@register.filter
def todatetime(value):
    return datetime(*time.localtime(time.mktime(value))[:6])

@register.tag
def rssplug(parser, token):
    try:
        tag_name, address, template = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('%s tag requires 2 arguments' % token.split_contents()[0])

    return RssPlugNode(address, template)

def resolve(var_or_value, ctx):
    if var_or_value[0] == '"':
        return var_or_value[1:-1]

    return ctx.resolve_variable(var_or_value)

class RssPlugNode(template.Node):
    def __init__(self, address, templ):
        self.address = address
        self.templ = templ

    def rss(self, addr):
        ckey = 'rssplug_%s' % addr
        data = cache.get(ckey)
        if data:
            return data
        data = feedparser.parse(addr)
        cache.set(ckey, data)
        return data

    def render(self, context):
        address = resolve(self.address, context)
        tmpl = resolve(self.templ, context)
        t = template.loader.get_template(tmpl)
        return ''.join([t.render(template.Context({ 'item': item })) for item in self.rss(address).entries])
